/*
 * Copyright (c) 2022 NetEase, Inc. All rights reserved.
 * Use of this source code is governed by a MIT license that can be
 * found in the LICENSE file.
 */

package com.netease.yunxin.flutter.plugins.roomkit

import android.content.Intent
import androidx.annotation.NonNull
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.PluginRegistry

/** RoomKitPlugin */
class RoomKitPlugin : FlutterPlugin, ActivityAware {

    private val activityResultLauncher = ActivityResultLauncher()

    private val platforms = listOf(
        RoomKitPlatform(),
        AuthServicePlatform(),
        RoomServicePlatform(),
        RoomContextPlatform(),
        RoomChatControllerPlatform(),
        RoomRtcControllerPlatform(activityResultLauncher),
        PreviewRoomRtcControllerPlatform(),
        RoomWhiteboardControllerPlatform(),
        RoomRtcRenderPlatform(),
        RoomLiveControllerPlatform(),
        MessageServicePlatform(),
        RoomSeatControllerPlatform()
    )

    override fun onAttachedToEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        platforms.forEach { it.onAttachedToEngine(binding) }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        platforms.forEach { it.onDetachedFromEngine(binding) }
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        activityResultLauncher.onAttachedToActivity(binding)
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        activityResultLauncher.onAttachedToActivity(binding)
    }

    override fun onDetachedFromActivityForConfigChanges() {
        activityResultLauncher.onDetachedFromActivity()
    }

    override fun onDetachedFromActivity() {
        activityResultLauncher.onDetachedFromActivity()
    }
}

class ActivityResultLauncher : PluginRegistry.ActivityResultListener {

    private var activityBinding: ActivityPluginBinding? = null
    private val resultCallbacks = hashSetOf<PluginRegistry.ActivityResultListener>()

    internal fun onAttachedToActivity(binding: ActivityPluginBinding) {
        activityBinding = binding
        binding.addActivityResultListener(this)
    }

    internal fun onDetachedFromActivity() {
        activityBinding!!.removeActivityResultListener(this)
        activityBinding = null
    }

    fun launch(intent: Intent, requestCode: Int): Boolean {
        val act = activityBinding?.activity
        if (act != null) {
            act.startActivityForResult(intent, requestCode)
            return true
        }
        return false
    }

    fun addActivityResultListener(listener: PluginRegistry.ActivityResultListener) {
        resultCallbacks.add(listener)
    }

    fun removeActivityResultListener(listener: PluginRegistry.ActivityResultListener) {
        resultCallbacks.remove(listener)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
        return resultCallbacks.any {
            it.onActivityResult(requestCode, resultCode, data)
        }
    }
}
