/*
 * Copyright (c) 2022 NetEase, Inc. All rights reserved.
 * Use of this source code is governed by a MIT license that can be
 * found in the LICENSE file.
 */

package com.netease.yunxin.flutter.plugins.roomkit.whiteboard

import android.content.Context
import com.netease.yunxin.kit.roomkit.api.NERoomKit
import com.netease.yunxin.kit.roomkit.api.service.NERoomService
import com.netease.yunxin.kit.roomkit.api.view.NEWhiteboardView
import io.flutter.plugin.common.StandardMessageCodec
import io.flutter.plugin.platform.PlatformView
import io.flutter.plugin.platform.PlatformViewFactory

internal class WhiteboardViewPlatform(
    context: Context,
    id: Int,
    creationParams: Map<String?, Any?>?
) :
    PlatformView {
    private var whiteboardView: NEWhiteboardView = NEWhiteboardView(context)
    private var roomUuid: String = creationParams?.get("roomUuid") as String

    init {
        (creationParams?.get("backgroundColor") as Int?)?.let { color ->
            whiteboardView.setBackgroundColor(color)
        }
    }

    override fun getView(): NEWhiteboardView {
        return whiteboardView
    }

    override fun dispose() {
        NERoomKit.instance.getService(NERoomService::class.java)
            .getRoomContext(roomUuid)?.whiteboardController?.resetWhiteboardCanvas(whiteboardView)
    }
}

class WhiteboardViewFactory : PlatformViewFactory(StandardMessageCodec.INSTANCE) {
    override fun create(context: Context?, viewId: Int, args: Any?): PlatformView {
        checkNotNull(context) { "Unable to create WhiteboardView without context" }
        val creationParams = args as Map<String?, Any?>?
        val roomUuid = creationParams?.get("roomUuid") as String
        val whiteboardView = WhiteboardViewPlatform(context, viewId, creationParams)
        NERoomKit.instance.getService(NERoomService::class.java)
            .getRoomContext(roomUuid)?.whiteboardController?.setupWhiteboardCanvas(whiteboardView.view)
        return whiteboardView
    }
}
