// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'dart:async';

import 'package:netease_roomkit/netease_roomkit.dart';

class HawkNERoomContextEx {
  static Future<dynamic> handleHawkMethodCall(
      Object? obj,
      String method,
      List<Map> params,
      Map<Type, Future<dynamic> Function(Map)> factory) async {
    if (method == 'addEventCallbackEx') {
      final callbackParam = params[0]['callback'] as Map?;

      final stream = StreamController<Map>();
      final data = Map();
      void notifyData(String key, List<dynamic> value) {
        if (callbackParam != null &&
            callbackParam.isNotEmpty &&
            !callbackParam.containsKey(key)) {
          return;
        }
        data[key] = value.length == 1
            ? value.single
            : value.fold<Map>(<String, dynamic>{}, (map, element) {
                map['arg${map.length}'] = element;
                return map;
              });
        stream.add(data);
      }

      final callback = NERoomEventCallback(
        memberPropertiesDeleted: (member, properties) =>
            notifyData('onMemberPropertiesDeleted', [member, properties]),
        memberPropertiesChanged: (member, properties) =>
            notifyData('onMemberPropertiesChanged', [member, properties]),
        memberJoinRoom: (members) => notifyData('onMemberJoinRoom', [members]),
        memberLeaveRoom: (members) =>
            notifyData('onMemberLeaveRoom', [members]),
        memberJoinChatroom: (members) =>
            notifyData('onMemberJoinChatroom', [members]),
        memberLeaveChatroom: (members) =>
            notifyData('onMemberLeaveChatroom', [members]),
        memberJoinRtcChannel: (members) =>
            notifyData('onMemberJoinRtcChannel', [members]),
        memberLeaveRtcChannel: (members) =>
            notifyData('onMemberLeaveRtcChannel', [members]),
        roomPropertiesChanged: (properties) =>
            notifyData('onRoomPropertiesChanged', [properties]),
        roomPropertiesDeleted: (properties) =>
            notifyData('onRoomPropertiesDeleted', [properties]),
        roomEnd: (reason) => notifyData('onRoomEnded', [reason]),
        memberNameChanged: (member, name) =>
            notifyData('onMemberNameChanged', [member, name]),
        memberAudioMuteChanged: (member, mute, operateBy) =>
            notifyData('onMemberAudioMuteChanged', [member, mute, operateBy]),
        memberVideoMuteChanged: (member, mute, operateBy) =>
            notifyData('onMemberVideoMuteChanged', [member, mute, operateBy]),
        memberScreenShareStateChanged: (member, isSharing, operateBy) =>
            notifyData('onMemberScreenShareStateChanged',
                [member, isSharing, operateBy]),
        memberWhiteboardShareStateChanged: (member, isSharing, operateBy) =>
            notifyData('onMemberWhiteboardStateChanged',
                [member, isSharing, operateBy]),
        rtcChannelError: (code) => notifyData('onRtcChannelError', [code]),
        chatroomMessagesReceived: (messages) =>
            notifyData('onReceiveChatroomMessages', [messages]),
        memberRoleChanged: (member, oldRole, newRole) =>
            notifyData('onMemberRoleChanged', [member, oldRole, newRole]),
        roomLockStateChanged: (isLocked) =>
            notifyData('onRoomLockStateChanged', [isLocked]),
        liveStateChanged: (state) =>
            notifyData('onRoomLiveStateChanged', [state]),
        rtcVirtualBackgroundSourceEnabled: (enabled, reason) => notifyData(
            'onRtcVirtualBackgroundSourceEnabled', [enabled, reason]),
        chatroomMessageAttachmentProgress: (messageUuid, transferred, total) =>
            notifyData('onChatroomMessageAttachmentProgress',
                [messageUuid, transferred, total]),
        rtcAudioMixingStateChanged: (reason) =>
            notifyData('onRtcAudioMixingStateChanged', [reason]),
        rtcAudioOutputDeviceChanged: (device) =>
            notifyData('onRtcAudioOutputDeviceChanged', [device]),
        rtcLocalAudioVolumeIndication: (volume, enableVad) =>
            notifyData('onRtcLocalAudioVolumeIndication', [volume, enableVad]),
        rtcRemoteAudioVolumeIndication: (volumes, totalVolume) => notifyData(
            'onRtcRemoteAudioVolumeIndication', [volumes, totalVolume]),
        whiteboardError: (code, msg) =>
            notifyData('onWhiteboardError', [code, msg]),
      );

      stream.onCancel = () {
        stream.close();
        (obj as NERoomContext).removeEventCallback(callback);
      };
      (obj as NERoomContext).addEventCallback(callback);
      return [stream.stream];
    }
  }
}
