// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'dart:async';

import 'package:netease_roomkit/netease_roomkit.dart';

class HawkNERoomSeatControllerEx {
  static Future<dynamic> handleHawkMethodCall(
      Object? obj,
      String method,
      List<Map> params,
      Map<Type, Future<dynamic> Function(Map)> factory) async {
    if (method == 'addEventCallbackEx') {
      final callbackParam = params[0]['callback'] as Map?;

      final stream = StreamController<Map>();
      final data = Map();
      void notifyData(String key, List<dynamic> value) {
        if (callbackParam != null &&
            callbackParam.isNotEmpty &&
            !callbackParam.containsKey(key)) {
          return;
        }
        data[key] = value.length == 1
            ? value.single
            : value.fold<Map>(
                <String, dynamic>{},
                (map, element) {
                  map['arg${map.length}'] = element;
                  return map;
                },
              );
        stream.add(data);
      }

      final callback = NESeatEventCallback(
        seatManagerAddedCallback: (value) =>
            notifyData('onSeatManagerAdded', [value]),
        seatManagerRemovedCallback: (value) =>
            notifyData('onSeatManagerRemoved', [value]),
        seatRequestSubmittedCallback: (index, user) =>
            notifyData('onSeatRequestSubmitted', [index, user]),
        seatRequestCancelledCallback: (index, user) =>
            notifyData('onSeatRequestCancelled', [index, user]),
        seatRequestApprovedCallback:
            (seatIndex, user, operateBy, isAutoAgree) => notifyData(
                'onSeatRequestApproved',
                [seatIndex, user, operateBy, isAutoAgree]),
        seatRequestRejectedCallback: (seatIndex, user, operateBy) =>
            notifyData('onSeatRequestRejected', [seatIndex, user, operateBy]),
        seatInvitationReceivedCallback: (seatIndex, user, operateBy) =>
            notifyData(
                'onSeatInvitationReceived', [seatIndex, user, operateBy]),
        seatInvitationCancelledCallback: (seatIndex, user, operateBy) =>
            notifyData(
                'onSeatInvitationCancelled', [seatIndex, user, operateBy]),
        seatInvitationAcceptedCallback: (seatIndex, user, isAutoAgree) =>
            notifyData(
                'onSeatInvitationAccepted', [seatIndex, user, isAutoAgree]),
        seatInvitationRejectedCallback: (seatIndex, user) =>
            notifyData('onSeatInvitationRejected', [seatIndex, user]),
        seatLeaveCallback: (seatIndex, user) =>
            notifyData('onSeatLeave', [seatIndex, user]),
        seatKickedCallback: (seatIndex, user, operateBy) =>
            notifyData('onSeatKicked', [seatIndex, user, operateBy]),
        seatListChangedCallback: (value) =>
            notifyData('onSeatListChanged', [value]),
      );

      stream.onCancel = () {
        stream.close();
        (obj as NERoomSeatController).removeEventCallback(callback);
      };
      (obj as NERoomSeatController).addEventCallback(callback);
      return [stream.stream];
    }

    return null;
  }
}
