// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:hawk/hawk.dart';
import 'package:netease_roomkit/netease_roomkit.dart';
import 'package:netease_common/netease_common.dart';

import 'case/netease_roomkit.hawk.dart';
import 'case/room_listener_case.dart';
import 'case/seat_listener_case.dart';

void main() async {
  final handler =
      HawkMethodCallDispatcher(resultTransformer: (String method, result) {
    if (result is NEResult) {
      return ResultBean(
          code: result.code, message: result.msg, data: result.data);
    }
    return ResultBean.success(message: method, data: result);
  });
  handler.registerMethodCallHandler(
    'NERoomKit',
    instanceGetter: () async => NERoomKit.instance,
    handlers: [HawkNERoomKit.handleHawkMethodCall],
  );
  handler.registerMethodCallHandler(
    'NEAuthService',
    instanceGetter: () async => NERoomKit.instance.authService,
    handlers: [HawkNEAuthService.handleHawkMethodCall],
  );
  handler.registerMethodCallHandler(
    'NERoomService',
    instanceGetter: () async => NERoomKit.instance.roomService,
    handlers: [HawkNERoomService.handleHawkMethodCall],
  );
  handler.registerMethodCallHandler(
    'NEMessageChannelService',
    instanceGetter: () async => NERoomKit.instance.messageChannelService,
    handlers: [HawkNEMessageChannelService.handleHawkMethodCall],
  );
  handler.registerMethodCallHandler(
    'NERoomContext',
    instanceGetterWithParams: (params) async {
      return NERoomKit.instance.roomService
          .getRoomContext(params.last['roomUuid']);
    },
    handlers: [
      HawkNERoomContext.handleHawkMethodCall,
      HawkNERoomContextEx.handleHawkMethodCall
    ],
  );
  handler.registerMethodCallHandler(
    'NERoomChatController',
    instanceGetterWithParams: (params) async {
      return NERoomKit.instance.roomService
          .getRoomContext(params.last['roomUuid'])!
          .chatController;
    },
    handlers: [HawkNERoomChatController.handleHawkMethodCall],
  );
  handler.registerMethodCallHandler(
    'NERoomSeatController',
    instanceGetterWithParams: (params) async {
      return NERoomKit.instance.roomService
          .getRoomContext(params.last['roomUuid'])!
          .seatController;
    },
    handlers: [
      HawkNERoomSeatController.handleHawkMethodCall,
      HawkNERoomSeatControllerEx.handleHawkMethodCall
    ],
  );
  handler.registerMethodCallHandler(
    'NERoomRtcController',
    instanceGetterWithParams: (params) async {
      return NERoomKit.instance.roomService
          .getRoomContext(params.last['roomUuid'])!
          .rtcController;
    },
    handlers: [
      HawkNERoomRtcController.handleHawkMethodCall,
      HawkNERoomBaseRtcController.handleHawkMethodCall
    ],
  );
  handler.registerMethodCallHandler(
    'NERoomWhiteboardController',
    instanceGetterWithParams: (params) async {
      return NERoomKit.instance.roomService
          .getRoomContext(params.last['roomUuid'])!
          .whiteboardController;
    },
    handlers: [HawkNERoomWhiteboardController.handleHawkMethodCall],
  );
  handler.registerMethodCallHandler(
    'NERoomLiveController',
    instanceGetterWithParams: (params) async {
      return NERoomKit.instance.roomService
          .getRoomContext(params.last['roomUuid'])!
          .liveController;
    },
    handlers: [HawkNERoomLiveController.handleHawkMethodCall],
  );

  handler.registerTypeConverter<NEPreviewRoomParams>(
      fromMap: (map) async => NEPreviewRoomParams());
  handler.registerTypeConverter<NEPreviewRoomOptions>(
      fromMap: (map) async => NEPreviewRoomOptions());
  handler.registerTypeConverter<NERoomEventCallback>(
      fromMap: (map) async => NERoomEventCallback());
  handler.registerTypeConverter<NESeatEventCallback>(
      fromMap: (map) async => NESeatEventCallback());
  handler.registerTypeConverter<NERoomRtcStatsCallback>(
      fromMap: (map) async => NERoomRtcStatsCallback());

  IntegrationTestWidgetsFlutterBinding.ensureInitialized();

  group(
    'roomkit flutter',
    () => test('roomkit flutter test', () async {
      await IntegratedPermissionHelper.requestPermissions(
          IntegratedConfig.permissions);
      await IntegratedManager.instance.init(
        applicationName: 'roomkit',
        version: '0.0.6',
        packageId: 'com.netease.yunxin.flutter.roomkit.sampleapp',
        caseList: [handler],
      );
    }, timeout: Timeout(Duration(minutes: 60))),
  );
}
