// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:flutter/material.dart';
import 'package:netease_roomkit/netease_roomkit.dart';

class ChatPage extends StatefulWidget {
  final NERoomContext roomContext;

  const ChatPage({required this.roomContext, Key? key}) : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _ChatPageState();
  }
}

class _ChatPageState extends State<ChatPage> {
  late NERoomChatController _controller;
  String message = '';
  String receiverMessage = '';

  @override
  void initState() {
    super.initState();
    _controller = widget.roomContext.chatController;
    widget.roomContext.addEventCallback(
      NERoomEventCallback(
          chatroomMessagesReceived: _chatroomMessagesReceived,
          memberLeaveChatroom: _memberLeaveChatRoom,
          memberJoinChatroom: _memberJoinChatroom),
    );
  }

  /// 加入聊天室
  void _memberJoinChatroom(List<NERoomMember> members) {
    String joinMembers = '';
    for (var element in members) {
      joinMembers = "${element.name},$joinMembers";
    }
    showTip(joinMembers);
  }

  /// 离开聊天室
  void _memberLeaveChatRoom(List<NERoomMember> members) {
    String leaveMembers = '';
    for (var element in members) {
      leaveMembers = "${element.name},$leaveMembers";
    }
    showTip(leaveMembers);
  }

  //// 获取消息
  void _chatroomMessagesReceived(List<NERoomChatMessage> messages) {
    messages.map((e) => e as NERoomChatTextMessage).toList().forEach((element) {
      receiverMessage = "$receiverMessage,${element.text}";
    });
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text('聊天'),
        centerTitle: true,
      ),
      backgroundColor: Colors.white,
      body: Column(
        children: [
          ElevatedButton(
              child: const Text('加入聊天室'),
              onPressed: () {
                _controller.joinChatroom().then((value) => showTip(value.msg));
              }),
          ElevatedButton(
              child: const Text('离开聊天室'),
              onPressed: () {
                _controller.leaveChatroom().then((value) {
                  if (value.code == 0) {
                    showTip(value.msg ?? '离开聊天室成功');
                  }
                });
              }),
          TextField(
            decoration: const InputDecoration(
              labelText: "message content",
            ),
            onChanged: (value) {
              message = value;
            },
          ),
          ElevatedButton(
              child: const Text('发送消息'),
              onPressed: () {
                _controller.sendBroadcastTextMessage(message);
              }),
          const SizedBox(
            height: 16,
          ),
          Text(
            '收到的消息：$receiverMessage',
          )
        ],
      ),
    );
  }

  showTip(String? msg) {
    ScaffoldMessenger.of(context).showSnackBar(
      SnackBar(
        content: Text(msg ?? ''),
      ),
    );
  }
}
