// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:roomkit_example/chat_page.dart';
import 'package:roomkit_example/video_view_page.dart';
import 'package:roomkit_example/whiteboard_page.dart';
import 'package:netease_roomkit/netease_roomkit.dart';

/// 登陆后跟房间创建，预览 加入相关的UI
class CreateRoomPage extends StatefulWidget {
  const CreateRoomPage({Key? key}) : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _CreateRoomPageState();
  }
}

class _CreateRoomPageState extends State<CreateRoomPage> {
  String roomId = '';
  String roomName = '';
  String? roomRole;
  bool enableLive = true;
  bool enableChatRoom = true;
  bool enableRtc = true;
  bool enableWhiteboard = true;
  // bool enableSip = true;
  NERoomContext? _roomContext;
  late TextEditingController _controller;
  late NERoomEventCallback _callback;
  String roomProperty = '';

  @override
  void initState() {
    super.initState();
    _controller = TextEditingController(text: 'audience');
    _callback = NERoomEventCallback(
        roomEnd: (reason) {
          showTip("roomEnd reason:${reason.index}");
        },
        memberRoleChanged: _memberRoleChanged,
        memberJoinRoom: _memberJoinRoom,
        memberLeaveRoom: _memberLeaveRoom,
        memberLeaveRtcChannel: _memberLeaveRtcChannel,
        memberJoinRtcChannel: _memberJoinRtcChannel,
        rtcChannelError: _rtcChannelError,
        memberAudioMuteChanged: _memberAudioMuteChanged,
        memberVideoMuteChanged: _memberVideoMuteChanged,
        memberScreenShareStateChanged: _memberScreenShareStateChanged,
        memberWhiteboardShareStateChanged: (member, operateMember, isSharing) {
          showTip(
              'uuid:${member.uuid},name:${member.name},isSharing:$isSharing');
        },
        roomPropertiesDeleted: (value) {
          value.forEach((key, value) {
            showTip('key:$key,value:$value');
          });
        },
        roomPropertiesChanged: (value) {
          value.forEach((key, value) {
            showTip('key:$key,value:$value');
          });
        });
  }

  @override
  void dispose() {
    _roomContext?.removeEventCallback(_callback);
    super.dispose();
  }

  void _memberScreenShareStateChanged(
      NERoomMember member, bool isSharing, NERoomMember? operator) {
    showTip(
        '_memberScreenShareStateChanged member:${member.name}, isSharing:$isSharing');
  }

  void _memberVideoMuteChanged(
      NERoomMember member, bool mute, NERoomMember? operator) {
    showTip('_memberVideoMuteChanged member:${member.name}, mute:$mute');
  }

  void _memberAudioMuteChanged(
      NERoomMember member, bool mute, NERoomMember? operator) {
    showTip('_memberAudioMuteChanged member:${member.name}, mute:$mute');
  }

  void _rtcChannelError(int code) {
    showTip('_rtcChannelError code:$code');
  }

  void _memberJoinRtcChannel(List<NERoomMember> members) {
    String member = '';
    for (var element in members) {
      member = '$member,${element.name}';
    }
    showTip('_memberJoinRtcChannel:$member');
  }

  void _memberLeaveRtcChannel(List<NERoomMember> members) {
    String member = '';
    for (var element in members) {
      member = '$member,${element.name}';
    }
    showTip('_memberLeaveRtcChannel:$member');
  }

  void _memberLeaveRoom(List<NERoomMember> members) {
    String member = '';
    for (var element in members) {
      member = '$member,${element.name}';
    }
    showTip('_memberLeaveRoom:$member');
  }

  void _memberJoinRoom(List<NERoomMember> members) {
    String member = '';
    for (var element in members) {
      member = '$member,${element.name}';
    }
    showTip('_memberJoinRoom:$member');
  }

  void _memberRoleChanged(
      NERoomMember member, NERoomRole before, NERoomRole after) {
    showTip(
        'memberRoleChanged,before role:${before.name},after role:${after.name}');
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text('房间'),
        centerTitle: true,
      ),
      resizeToAvoidBottomInset: false,
      backgroundColor: Colors.white,
      body: GestureDetector(
        onTap: () => FocusScope.of(context).requestFocus(FocusNode()),
        child: SingleChildScrollView(
          child: Padding(
            padding: const EdgeInsets.all(8.0),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                TextField(
                  autofocus: true,
                  decoration: const InputDecoration(
                    labelText: "room id",
                  ),
                  onChanged: (value) {
                    roomId = value;
                  },
                ),
                TextField(
                  autofocus: true,
                  decoration: const InputDecoration(
                    labelText: "room name",
                  ),
                  onChanged: (value) {
                    roomName = value;
                  },
                ),
                TextField(
                  autofocus: true,
                  decoration: const InputDecoration(
                    labelText: "room role",
                  ),
                  onChanged: (value) {
                    roomRole = value;
                  },
                  controller: _controller,
                ),
                const SizedBox(
                  height: 12,
                ),
                const Text('创建房间参数'),
                const SizedBox(
                  height: 12,
                ),
                buildEnableLive(),
                Container(
                  height: 1,
                  color: Colors.grey,
                  margin: const EdgeInsets.only(top: 8, bottom: 4),
                ),
                buildEnableChatRoom(),
                Container(
                  height: 1,
                  color: Colors.grey,
                  margin: const EdgeInsets.only(top: 8, bottom: 4),
                ),
                buildEnableRtc(),
                Container(
                  height: 1,
                  color: Colors.grey,
                  margin: const EdgeInsets.only(top: 8, bottom: 4),
                ),
                buildEnableWhiteboard(),
                Container(
                  height: 1,
                  color: Colors.grey,
                  margin: const EdgeInsets.only(top: 8, bottom: 12),
                ),
                buildRoomOperate(),
                const SizedBox(
                  height: 8,
                ),
                buildSkipToFunctionPage(),
                const SizedBox(
                  height: 8,
                ),
                buildRoomOperate1(),
                const SizedBox(
                  height: 8,
                ),
                buildRoomOperate2(),
                const SizedBox(
                  height: 8,
                ),
                TextField(
                  autofocus: true,
                  decoration: const InputDecoration(
                    labelText: "room property",
                  ),
                  onChanged: (value) {
                    roomProperty = value;
                  },
                ),
                ElevatedButton(
                    child: const Text('删除属性'),
                    onPressed: () {
                      _roomContext?.deleteRoomProperty(roomProperty);
                    }),
              ],
            ),
          ),
        ),
      ),
    );
  }

  Widget buildEnableLive() {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: [
        const Text('enableRtc'),
        CupertinoSwitch(
          value: enableRtc,
          onChanged: (bool value) {
            setState(() {
              enableRtc = value;
            });
          },
        )
      ],
    );
  }

  Widget buildEnableChatRoom() {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: [
        const Text('enableChatRoom'),
        CupertinoSwitch(
          value: enableChatRoom,
          onChanged: (bool value) {
            setState(() {
              enableChatRoom = value;
            });
          },
        )
      ],
    );
  }

  Widget buildEnableRtc() {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: [
        const Text('enableRtc'),
        CupertinoSwitch(
          value: enableChatRoom,
          onChanged: (bool value) {
            setState(() {
              enableChatRoom = value;
            });
          },
        )
      ],
    );
  }

  Widget buildEnableWhiteboard() {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: [
        const Text('enableWhiteboard'),
        CupertinoSwitch(
          value: enableWhiteboard,
          onChanged: (bool value) {
            setState(() {
              enableWhiteboard = value;
            });
          },
        )
      ],
    );
  }

  //// 房间相关操作
  Widget buildRoomOperate() {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: [
        ElevatedButton(
            child: const Text('预览'),
            onPressed: () => Navigator.of(context).push(MaterialPageRoute(
                builder: (context) => VideoViewPage(roomUuid: roomId)))),
        ElevatedButton(
            child: const Text('创建房间'),
            onPressed: () {
              var params = NECreateRoomParams(
                  roomUuid: roomId, roomName: roomName, templateId: 21);
              var options = NECreateRoomOptions(
                enableLive: enableLive,
                enableRtc: enableRtc,
                enableChatroom: enableChatRoom,
                enableWhiteboard: enableWhiteboard,
                // enableSip: enableSip,
              );
              NERoomKit.instance.roomService
                  .createRoom(params, options)
                  .then((value) {
                ScaffoldMessenger.of(context).showSnackBar(
                  SnackBar(
                    content: Text('${value.msg}'),
                  ),
                );
              });
            }),
        ElevatedButton(
            child: const Text('加入房间'),
            onPressed: () {
              var params = NEJoinRoomParams(
                  roomUuid: roomId, userName: 'test', role: roomRole ?? 'host');
              var option = NEJoinRoomOptions();
              NERoomKit.instance.roomService
                  .joinRoom(params, option)
                  .then((value) {
                if (value.code == 0) {
                  _roomContext = value.data;
                  _roomContext!.addEventCallback(_callback);
                  showTip(value.msg);
                }
              });
            }),
      ],
    );
  }

  Widget buildSkipToFunctionPage() {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: [
        ElevatedButton(
            child: const Text('退出房间'),
            onPressed: () {
              if (_roomContext != null) {
                _roomContext!.leaveRoom().then((value) {
                  showTip(value.msg ?? '退出房间');
                  if (value.code == 0) {
                    _roomContext = null;
                  }
                });
              } else {
                showTip('please join room first');
              }
            }),
        ElevatedButton(
            child: const Text('结束'),
            onPressed: () {
              if (_roomContext != null) {
                _roomContext!.endRoom().then((value) {
                  showTip(value.msg ?? '结束房间');
                  if (value.code == 0) {
                    _roomContext = null;
                  }
                });
              } else {
                showTip('please join room first');
              }
            }),
        ElevatedButton(
            child: const Text('进入白板'),
            onPressed: () {
              if (_roomContext != null) {
                Navigator.of(context)
                    .push(MaterialPageRoute(builder: (context) {
                  return WhiteboardPage(roomContext: _roomContext!);
                }));
              } else {
                showTip('please join room first');
              }
            }),
        ElevatedButton(
            child: const Text('进入聊天室'),
            onPressed: () {
              if (_roomContext != null) {
                Navigator.of(context)
                    .push(MaterialPageRoute(builder: (context) {
                  return ChatPage(roomContext: _roomContext!);
                }));
              } else {
                showTip('please join room first');
              }
            }),
      ],
    );
  }

  Widget buildRoomOperate1() {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: [
        ElevatedButton(
            child: const Text('进入RTC'),
            onPressed: () {
              _roomContext?.rtcController.joinRtcChannel().then((value) {
                print('joinRtcChannel:${value.msg}');
              });
            }),
        ElevatedButton(
            child: const Text('离开RTC'),
            onPressed: () {
              _roomContext?.rtcController.leaveRtcChannel().then((value) {
                print('leaveRtcChannel:${value.msg}');
              });
            }),
        ElevatedButton(
            child: const Text('关闭音频'),
            onPressed: () {
              _roomContext?.rtcController.muteMyAudio().then((value) {
                print('muteMyAudio:${value.msg}');
              });
            }),
      ],
    );
  }

  Widget buildRoomOperate2() {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: [
        ElevatedButton(
            child: const Text('打开音频'),
            onPressed: () {
              _roomContext?.rtcController.unmuteMyAudio().then((value) {
                print('unmuteMyAudio:${value.msg}');
              });
            }),
        ElevatedButton(
            child: const Text('关闭视频'),
            onPressed: () {
              _roomContext?.rtcController.muteMyVideo().then((value) {
                print('muteMyVideo:${value.msg}');
              });
            }),
        ElevatedButton(
            child: const Text('打开视频'),
            onPressed: () {
              _roomContext?.rtcController.unmuteMyVideo().then((value) {
                print('unmuteMyVideo:${value.msg}');
              });
            }),
      ],
    );
  }

  showTip(String? msg) {
    ScaffoldMessenger.of(context).showSnackBar(
      SnackBar(
        content: Text(msg ?? ''),
      ),
    );
  }
}
