// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:flutter/material.dart';
import 'package:netease_roomkit/netease_roomkit.dart';

class WhiteboardPage extends StatefulWidget {
  final NERoomContext roomContext;

  const WhiteboardPage({required this.roomContext, Key? key}) : super(key: key);

  @override
  State<WhiteboardPage> createState() => _WhiteboardPageState();
}

class _WhiteboardPageState extends State<WhiteboardPage> {
  late NERoomWhiteboardController _controller;
  late NERoomEventCallback _callback;
  String uUid = '';
  bool isHasSharing = false;
  @override
  void initState() {
    super.initState();
    _controller = widget.roomContext.whiteboardController;
    _callback = NERoomEventCallback(
      memberWhiteboardShareStateChanged:
          (NERoomMember member, bool isSharing, NERoomMember? operator) {
        print('白板共享状态变更:$isSharing');
        isHasSharing = isSharing;
        setState(() {});
      },
    );
    widget.roomContext.addEventCallback(_callback);
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text('白板'),
        centerTitle: true,
      ),
      backgroundColor: Colors.white,
      body: Column(
        children: [
          buildWhiteboardOperate(),
          const SizedBox(
            height: 8,
          ),
          TextField(
            decoration: const InputDecoration(
              labelText: "whiteboard uid",
            ),
            onChanged: (value) {
              uUid = value;
            },
          ),
          const SizedBox(
            height: 8,
          ),
          const SizedBox(
            height: 8,
          ),
          if (isHasSharing)
            Expanded(flex: 1, child: _controller.createWhiteboardView()),
        ],
      ),
    );
  }

  buildWhiteboardOperate() {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceAround,
      children: [
        ElevatedButton(
            onPressed: () {
              _controller.startWhiteboardShare();
            },
            child: const Text('共享白板')),
        ElevatedButton(
            onPressed: () {
              _controller.stopWhiteboardShare();
            },
            child: const Text('停止共享白板')),
      ],
    );
  }

  showTip(String? msg) {
    ScaffoldMessenger.of(context).showSnackBar(
      SnackBar(
        content: Text(msg ?? ''),
      ),
    );
  }
}
