// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import Foundation
import NERoomKit

class FLTRoomContext: NSObject, FLTRoomContextApi {
  func handOverMyRoleRoomUuid(_ roomUuid: String?,
                              userUuid: String?,
                              completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid),
          let userUuid = userUuid else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.handOverMyRole(userUuid: userUuid) { code, str, _ in
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
    }
  }

  func changeMyNameRoomUuid(_ roomUuid: String?, name: String?,
                            completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid),
          let name = name else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.changeMyName(userName: name) { code, str, _ in
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
    }
  }

  func lockRoomRoomUuid(_ roomUuid: String?,
                        completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.lockRoom { code, str, _ in
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
    }
  }

  func unlockRoomRoomUuid(_ roomUuid: String?,
                          completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.unlockRoom { code, str, _ in
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
    }
  }

  func updateRoomPropertyRoomUuid(_ roomUuid: String?,
                                  key: String?,
                                  value: String?,
                                  associatedUuid: String?,
                                  completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                    -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid),
          let key = key,
          let value = value else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.updateRoomProperty(key: key,
                                   value: value,
                                   associatedUserUuid: associatedUuid) { code, str, response in
      let response = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(response, nil)
    }
  }

  func deleteRoomPropertyRoomUuid(_ roomUuid: String?,
                                  key: String?,
                                  completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                    -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid),
          let key = key else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.deleteRoomProperty(key: key) { code, str, _ in
      let response = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(response, nil)
    }
  }

  func updateMemberPropertyRoomUuid(_ roomUuid: String?,
                                    userUuid: String?,
                                    key: String?,
                                    value: String?,
                                    completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                      -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid),
          let userUuid = userUuid,
          let key = key,
          let value = value else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.updateMemberProperty(userUuid: userUuid,
                                     key: key,
                                     value: value) { code, str, _ in
      let response = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(response, nil)
    }
  }

  func deleteMemberPropertyRoomUuid(_ roomUuid: String?,
                                    userUuid: String?,
                                    key: String?,
                                    completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                      -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid),
          let userUuid = userUuid,
          let key = key else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.deleteMemberProperty(userUuid: userUuid,
                                     key: key) { code, str, _ in
      let response = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(response, nil)
    }
  }

  func changeMemberRoleRoomUuid(_ roomUuid: String?,
                                userUuid: String?,
                                role: String?,
                                completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                  -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid),
          let userUuid = userUuid,
          let role = role else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.changeMemberRole(userUuid: userUuid, role: role) { code, str, _ in
      let response = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(response, nil)
    }
  }

  func kickMemberOutRoomUuid(_ roomUuid: String?,
                             userUuid: String?,
                             completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid),
          let userUuid = userUuid else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.kickMemberOut(userUuid: userUuid) { code, str, _ in
      let response = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(response, nil)
    }
  }

  func leaveRoomRoomUuid(_ roomUuid: String?,
                         completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.leaveRoom { code, str, _ in
      let response = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(response, nil)
    }
  }

  func endRoomRoomUuid(_ roomUuid: String?,
                       isForce: NSNumber?,
                       completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.endRoom(isForce: isForce?.boolValue ?? false) { code, str, _ in
      let response = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(response, nil)
    }
  }
}

extension FLTRoomContext: FLTAssociativeWrapper {
  func onAttachedToEngine(_ registrar: FlutterPluginRegistrar) {
    FLTRoomContextApiSetup(registrar.messenger(), self)
  }

  func onDetachedFromEngine(_ registrar: FlutterPluginRegistrar) {
    FLTRoomContextApiSetup(registrar.messenger(), nil)
  }
}
