// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

#import "FLNativeViewFactory.h"
// #import <Flutter/Flutter.h>
// #import <NERoomKit/NERoomKit-Swift.h>
@import NERoomKit;

@interface FLNativeViewFactory ()
@property(nonatomic, strong) NSObject<FlutterBinaryMessenger> *messenger;

@end

@implementation FLNativeViewFactory
- (instancetype)initWithFlutterBinaryMessenger:(NSObject<FlutterBinaryMessenger> *)messenger {
  self = [super init];
  if (self) {
    self.messenger = messenger;
  }
  return self;
}
- (NSObject<FlutterMessageCodec> *)createArgsCodec {
  return [FlutterStandardMessageCodec sharedInstance];
}

- (NSObject<FlutterPlatformView> *)createWithFrame:(CGRect)frame
                                    viewIdentifier:(int64_t)viewId
                                         arguments:(id)args {
  return [[FLTWhiteboardView alloc] initWithFrame:frame
                                   viewIdentifier:viewId
                                        arguments:args
                                  binaryMessenger:self.messenger];
}
@end

@implementation FLTWhiteboardView {
  NEWhiteboardView *_whiteboardView;
  NSDictionary *_paramsDictionary;
}

- (instancetype)initWithFrame:(CGRect)frame
               viewIdentifier:(int64_t)viewId
                    arguments:(id _Nullable)args
              binaryMessenger:(NSObject<FlutterBinaryMessenger> *)messenger {
  if (self = [super init]) {
    _whiteboardView = [[NEWhiteboardView alloc] init];

    if ([args isKindOfClass:NSDictionary.class]) {
      _paramsDictionary = args;
      NERoomContext *context =
          [NERoomKit.shared.roomService getRoomContextWithRoomUuid:_paramsDictionary[@"roomUuid"]];
      if (context) {
        [context.whiteboardController setupWhiteboardCanvasWithView:_whiteboardView];
      }
    }
  }
  return self;
}
- (void)dealloc {
  NERoomContext *context =
      [NERoomKit.shared.roomService getRoomContextWithRoomUuid:_paramsDictionary[@"roomUuid"]];
  if (context) {
    [context.whiteboardController resetWhiteboardCanvasWithView:_whiteboardView];
  }
}

- (UIView *)view {
  return _whiteboardView;
}

@end
