// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import Foundation

class FLTPreviewRoomRtcController: NSObject, FLTPreviewRoomRtcControllerApi {
  func enableVirtualBackgroundEnabled(_ enabled: NSNumber?,
                                      backgroundSource: FLTNERoomVirtualBackgroundSource?,
                                      completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                        -> Void) {
    guard let enabled = enabled,
          let context = NERoomKit.shared().roomService.getPreviewRoomContext() else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    var source: NERoomVirtualBackgroundSource?
    if let backData = backgroundSource {
      source = NERoomVirtualBackgroundSource()
      source?
        .backgroundSourceType = NERoomVirtualBackgroundSourceType(rawValue: backData
          .backgroundSourceType.uintValue) ?? .color
      source?.color = backData.color.uintValue
      source?.source = backData.source
      source?
        .blurDegree = NERoomBlurDegree(rawValue: backData.blurDegree.uintValue) ?? .high
    }
    let code = context.previewController.enableVirtualBackground(
      enable: enabled.boolValue,
      backgroundSource: source
    )
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )

    completion(simpleResponse, nil)
  }

  func switchCamera(completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let context = NERoomKit.shared().roomService.getPreviewRoomContext() else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = context.previewController.switchCamera()
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func startPreview(completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let context = NERoomKit.shared().roomService.getPreviewRoomContext() else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = context.previewController.startPreview()
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func stopPreviewReleaseRtc(_ releaseRtc: NSNumber, completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let context = NERoomKit.shared().roomService.getPreviewRoomContext() else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = context.previewController.stopPreview(releaseRtc: releaseRtc.boolValue)
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func addBeautyFilterPath(_ path: String?,
                           completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let context = NERoomKit.shared().roomService.getPreviewRoomContext(),
          let path = path else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = context.previewController.addBeautyFilter(path: path)
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func addBeautyStickerPath(_ path: String?,
                            completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let context = NERoomKit.shared().roomService.getPreviewRoomContext(),
          let path = path else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = context.previewController.addBeautySticker(path: path)
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func enableBeautyIsOpenBeauty(_ isOpenBeauty: NSNumber?,
                                completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                  -> Void) {
    guard let context = NERoomKit.shared().roomService.getPreviewRoomContext(),
          let isOpenBeauty = isOpenBeauty else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = context.previewController.enableBeauty(isOpenBeauty: isOpenBeauty.boolValue)
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func removeBeautyFilter(completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let context = NERoomKit.shared().roomService.getPreviewRoomContext() else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = context.previewController.removeBeautyFilter()
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func removeBeautySticker(completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let context = NERoomKit.shared().roomService.getPreviewRoomContext() else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = context.previewController.removeBeautySticker()
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func startBeauty(completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let context = NERoomKit.shared().roomService.getPreviewRoomContext() else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = context.previewController.startBeauty()
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func stopBeauty(completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let context = NERoomKit.shared().roomService.getPreviewRoomContext() else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = context.previewController.stopBeauty()
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func setBeautyFilterLevelLevel(_ level: NSNumber?,
                                 completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                   -> Void) {
    guard let context = NERoomKit.shared().roomService.getPreviewRoomContext(),
          let level = level else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = context.previewController.setBeautyFilterLevel(level: level.floatValue)
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func setBeautyEffectBeautyType(_ beautyType: NSNumber?, level: NSNumber?,
                                 completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                   -> Void) {
    guard let context = NERoomKit.shared().roomService.getPreviewRoomContext(),
          let beautyType = beautyType,
          let level = level else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }

    let code = context.previewController.setBeautyEffect(
      beautyType: NERoomBeautyEffectType(rawValue: beautyType.intValue - 1) ?? .whiteTeeth,
      level: level.floatValue
    )
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }
}

extension FLTPreviewRoomRtcController: FLTAssociativeWrapper {
  func onAttachedToEngine(_ registrar: FlutterPluginRegistrar) {
    FLTPreviewRoomRtcControllerApiSetup(registrar.messenger(), self)
  }

  func onDetachedFromEngine(_ registrar: FlutterPluginRegistrar) {
    FLTPreviewRoomRtcControllerApiSetup(registrar.messenger(), nil)
  }
}
