// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import Foundation
import NERoomKit

class FLTRoomRtcController: NSObject, FLTRoomRtcControllerApi {
  func enableMediaPubRoomUuid(_ roomUuid: String?, mediaType: NSNumber?, enable: NSNumber?, completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let mediaType = mediaType,
          let enable = enable,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = roomContext.rtcController.enableMediaPub(mediaType: NERoomRtcMediaPublishType(rawValue: mediaType.uintValue) ?? .audio, enable: enable.boolValue)
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func enableAudioVolumeIndicationRoomUuid(_ roomUuid: String?, enable: NSNumber?, interval: NSNumber?, vad: NSNumber?, completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let enable = enable,
          let interval = interval,
          let vad = vad,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = roomContext.rtcController.enableAudioVolumeIndication(
      enable: enable.boolValue,
      interval: interval.intValue,
      vad: vad.boolValue
    )
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func pauseLocalAudioRecordingRoomUuid(_ roomUuid: String?, completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = roomContext.rtcController.pauseLocalAudioRecording()
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func resumeLocalAudioRecordingRoomUuid(_ roomUuid: String?, completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = roomContext.rtcController.resumeLocalAudioRecording()
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func pauseLocalVideoCaptureRoomUuid(_ roomUuid: String?, completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = roomContext.rtcController.pauseLocalVideoCapture()
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func resumeLocalVideoCaptureRoomUuid(_ roomUuid: String?, completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = roomContext.rtcController.resumeLocalVideoCapture()
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func adjustPlaybackSignalVolumeRoomUuid(_ roomUuid: String?, volume: NSNumber?, completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid),
          let volume = volume
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = roomContext.rtcController.adjustPlaybackSignalVolume(volume.intValue)
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func pauseAudioMixingRoomUuid(_ roomUuid: String?,
                                completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                  -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }

    let code = roomContext.rtcController.pauseAudioMixing()
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func adjustRecordingSignalVolumeRoomUuid(_ roomUuid: String?, volume: NSNumber?,
                                           completion: @escaping (FLTSimpleResponse?,
                                                                  FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let volume = volume,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = roomContext.rtcController.adjustRecordingSignalVolume(volume: volume.uint32Value)
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func resumeAudioMixingRoomUuid(_ roomUuid: String?,
                                 completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                   -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = roomContext.rtcController.resumeAudioMixing()
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func setRecordDeviceMuteRoomUuid(_ roomUuid: String?, mute: NSNumber?,
                                   completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                     -> Void) {
    guard let roomUuid = roomUuid,
          let mute = mute,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = roomContext.rtcController.setRecordDeviceMute(muted: mute.boolValue)
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func setClientRoleRoomUuid(_ roomUuid: String?, role: NSNumber?,
                             completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let role = role,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = roomContext.rtcController.setClientRole(NERoomRtcClientRole(rawValue: role.intValue) ?? .audience)
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func setLocalAudioProfileRoomUuid(_ roomUuid: String?, profile: NSNumber?, scenario: NSNumber?,
                                    completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                      -> Void) {
    guard let roomUuid = roomUuid,
          let profile = profile,
          let scenario = scenario,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = roomContext.rtcController.setAudioProfile(NERoomRtcAudioProfileType(rawValue: profile.intValue) ?? .default, scenario: NERoomRtcAudioScenarioType(rawValue: scenario.intValue) ?? .default)

//      let code = roomContext.rtcController.setClientRole(NERoomRtcClientRole(rawValue: role.intValue) ?? .audience)
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)

//  }
  }

  func setChannelProfileRoomUuid(_ roomUuid: String?, value: NSNumber?,
                                 completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                   -> Void) {
    guard let roomUuid = roomUuid,
          let value = value,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = roomContext.rtcController.setChannelProfile(NERoomRtcChannelProfileType(rawValue: value.intValue) ?? .communication)
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func addBeautyFilterRoomUuid(_ roomUuid: String?,
                               path: String?,
                               completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                 -> Void) {
    guard let roomUuid = roomUuid,
          let path = path,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      return
    }
    let code = roomContext.rtcController.addBeautyFilter(path: path)
    let response = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(response, nil)
  }

  func addBeautyStickerRoomUuid(_ roomUuid: String?,
                                path: String?,
                                completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                  -> Void) {
    guard let roomUuid = roomUuid,
          let path = path,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      return
    }
    let code = roomContext.rtcController.addBeautySticker(path: path)
    let response = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(response, nil)
  }

  func enableBeautyRoomUuid(_ roomUuid: String?,
                            isOpenBeauty: NSNumber?,
                            completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let isOpenBeauty = isOpenBeauty,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      return
    }
    let code = roomContext.rtcController.enableBeauty(isOpenBeauty: isOpenBeauty.boolValue)
    let response = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(response, nil)
  }

  func removeBeautyFilterRoomUuid(_ roomUuid: String?,
                                  completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                    -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      return
    }
    let code = roomContext.rtcController.removeBeautyFilter()
    let response = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(response, nil)
  }

  func removeBeautyStickerRoomUuid(_ roomUuid: String?,
                                   completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                     -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      return
    }
    let code = roomContext.rtcController.removeBeautySticker()
    let response = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(response, nil)
  }

  func startBeautyRoomUuid(_ roomUuid: String?,
                           completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      return
    }
    let code = roomContext.rtcController.startBeauty()
    let response = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(response, nil)
  }

  func stopBeautyRoomUuid(_ roomUuid: String?,
                          completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      return
    }
    let code = roomContext.rtcController.stopBeauty()
    let response = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(response, nil)
  }

  func setBeautyFilterLevelRoomUuid(_ roomUuid: String?,
                                    level: NSNumber?,
                                    completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                      -> Void) {
    guard let roomUuid = roomUuid,
          let level = level,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      return
    }
    let code = roomContext.rtcController.setBeautyFilterLevel(level: level.floatValue)
    let response = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(response, nil)
  }

  func setBeautyEffectRoomUuid(_ roomUuid: String?,
                               beautyType: NSNumber?,
                               level: NSNumber?,
                               completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                 -> Void) {
    guard let roomUuid = roomUuid,
          let beautyType = beautyType,
          let level = level,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      return
    }
    let code = roomContext.rtcController.setBeautyEffect(
      beautyType: NERoomBeautyEffectType(rawValue: beautyType.intValue) ?? .unknown,
      level: level.floatValue
    )
    let response = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(response, nil)
  }

  func adjustUserPlaybackSignalVolumeRoomUuid(_ roomUuid: String?, userUuid: String?,
                                              volume: NSNumber?,
                                              completion: @escaping (FLTSimpleResponse?,
                                                                     FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid),
          let userUuid = userUuid,
          let volume = volume else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = roomContext.rtcController.adjustUserPlaybackSignalVolume(
      userUuid: userUuid,
      volume: volume.uint32Value
    )
    let response = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(response, nil)
  }

  func enableEarBackRoomUuid(_ roomUuid: String?, enable: NSNumber?, volume: NSNumber?,
                             completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    if let enable = enable?.boolValue,
       enable {
      let code = roomContext.rtcController
        .enableEarback(volume: volume?.uint32Value ?? 80)
      let response = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: nil,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(response, nil)
    } else {
      let code = roomContext.rtcController.disableEarback()
      let response = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: nil,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(response, nil)
    }
  }

  func startAudioMixingRoomUuid(_ roomUuid: String?, option: FLTCreateAudioMixingOption?,
                                completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                  -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }

    let o = NERoomCreateAudioMixingOption()
    o.playbackVolume = option?.playbackVolume.intValue ?? 100
    o.path = option?.path ?? ""
    o.loopCount = option?.loopCount.intValue ?? 1
    o.sendEnabled = option?.sendEnabled.boolValue ?? true
    o.sendVolume = option?.sendVolume.intValue ?? 100
    o.playbackEnabled = option?.playbackEnabled.boolValue ?? true
    o.playbackVolume = option?.playbackVolume.intValue ?? 0
    o.startTimeStamp = option?.startTimeStamp.int64Value ?? 0
    o.sendWithAudioType = option?.sendWithAudioType == FLTAudioStreamType.audioStreamTypeMain ? NERoomAudioStreamType.main : NERoomAudioStreamType.sub
    let code = roomContext.rtcController.startAudioMixing(option: o)
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func playEffectRoomUuid(_ roomUuid: String?, id: NSNumber?, option: FLTCreateAudioEffectOption?,
                          completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid),
          let effectId = id?.uint32Value else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let o = NERoomCreateAudioEffectOption()
    o.playbackVolume = option?.playbackVolume.intValue ?? 100
    o.path = option?.path ?? ""
    o.loopCount = option?.loopCount.intValue ?? 1
    o.sendEnabled = option?.sendEnabled.boolValue ?? true
    o.sendVolume = option?.sendVolume.intValue ?? 100
    o.playbackEnabled = option?.playbackEnabled.boolValue ?? true
    let code = roomContext.rtcController.playEffect(effectId: effectId, option: o)
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func stopAudioMixingRoomUuid(_ roomUuid: String?,
                               completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                 -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = roomContext.rtcController.stopAudioMixing()
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func stopEffectRoomUuid(_ roomUuid: String?, id: NSNumber?,
                          completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid),
          let effectId = id?.uint32Value else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = roomContext.rtcController.stopEffect(effectId: effectId)
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func setAudioMixingSendVolumeRoomUuid(_ roomUuid: String?, volume: NSNumber?,
                                        completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                          -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = roomContext.rtcController
      .setAudioMixingSendVolume(volume: volume?.uint32Value ?? 100)
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func setAudioMixingPlaybackVolumeRoomUuid(_ roomUuid: String?, volume: NSNumber?,
                                            completion: @escaping (FLTSimpleResponse?,
                                                                   FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = roomContext.rtcController
      .setAudioMixingPlaybackVolume(volume: volume?.uint32Value ?? 100)
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func setEffectSendVolumeRoomUuid(_ roomUuid: String?, id: NSNumber?, volume: NSNumber?,
                                   completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                     -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid),
          let effectId = id?.uint32Value else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = roomContext.rtcController.setEffectSendVolume(
      effectId: effectId,
      volume: volume?.uint32Value ?? 100
    )
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func setEffectPlaybackVolumeRoomUuid(_ roomUuid: String?, id: NSNumber?, volume: NSNumber?,
                                       completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                         -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid),
          let effectId = id?.uint32Value else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = roomContext.rtcController.setEffectPlaybackVolume(
      effectId: effectId,
      volume: volume?.uint32Value ?? 100
    )
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func stopAllEffectsRoomUuid(_ roomUuid: String?,
                              completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = roomContext.rtcController.stopAllEffects()
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func startRtcChannelMediaRelayRoomUuid(_ roomUuid: String?,
                                         completion: @escaping (FLTSimpleResponse?,
                                                                FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.rtcController.startChannelMediaRelay { code, msg, _ in
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: msg,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
    }
  }

  func stopRtcChannelMediaRelayRoomUuid(_ roomUuid: String?,
                                        completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                          -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.rtcController.stopChannelMediaRelay { code, msg, _ in
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: msg,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
    }
  }

  func setSpeakerphoneOnRoomUuid(_ roomUuid: String?,
                                 on: NSNumber?,
                                 completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                   -> Void) {
    guard let roomUuid = roomUuid,
          let on = on,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = roomContext.rtcController.setSpeakerphoneOn(on: on.boolValue)
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func isSpeakerphone(onRoomUuid roomUuid: String?,
                      completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let isOn = roomContext.rtcController.isSpeakerphoneOn()
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: isOn),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func stopAudioDumpRoomUuid(_ roomUuid: String?,
                             completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = roomContext.rtcController.stopAudioDump()
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func startAudioDumpRoomUuid(_ roomUuid: String?,
                              type: NSNumber?,
                              completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let type = type,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }

    let code = roomContext.rtcController
      .startAudioDump(type: NEAudioDumpType(rawValue: type.intValue) ?? .pcm)
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func enableAudioVolumeIndicationRoomUuid(_ roomUuid: String?,
                                           enable: NSNumber?,
                                           interval: NSNumber?,
                                           completion: @escaping (FLTSimpleResponse?,
                                                                  FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let enable = enable,
          let interval = interval,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = roomContext.rtcController.enableAudioVolumeIndication(
      enable: enable.boolValue,
      interval: interval.intValue
    )
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func startScreenShareRequest(_ request: FLTStartScreenShareRequest?,
                               completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                 -> Void) {
    guard let request = request,
          let appGroup = request.iosAppGroup,
          let roomContext = NERoomKit.shared().roomService
          .getRoomContext(roomUuid: request.roomUuid) else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.rtcController.setupShareKit(appGroup: appGroup)
    roomContext.rtcController.startScreenShare { code, str, _ in
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
    }
  }

  func muteMemberAudioRoomUuid(_ roomUuid: String?,
                               uuid: String?,
                               completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                 -> Void) {
    guard let roomUuid = roomUuid,
          let userUuid = uuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.rtcController.muteMemberAudio(userUuid: userUuid) { code, str, _ in
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
    }
  }

  func unmuteMemberAudioRoomUuid(_ roomUuid: String?,
                                 uuid: String?,
                                 completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                   -> Void) {
    guard let roomUuid = roomUuid,
          let userUuid = uuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.rtcController.unmuteMemberAudio(userUuid: userUuid) { code, str, _ in
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
    }
  }

  func muteMemberVideoRoomUuid(_ roomUuid: String?,
                               uuid: String?,
                               completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                 -> Void) {
    guard let roomUuid = roomUuid,
          let userUuid = uuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.rtcController.muteMemberVideo(userUuid: userUuid) { code, str, _ in
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
    }
  }

  func unmuteMemberVideoRoomUuid(_ roomUuid: String?,
                                 uuid: String?,
                                 completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                   -> Void) {
    guard let roomUuid = roomUuid,
          let userUuid = uuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.rtcController.unmuteMemberVideo(userUuid: userUuid) { code, str, _ in
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
    }
  }

  func stopMemberScreenShareRoomUuid(_ roomUuid: String?,
                                     uuid: String?,
                                     completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                       -> Void) {
    guard let roomUuid = roomUuid,
          let userUuid = uuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.rtcController.stopMemberScreenShare(userUuid: userUuid) { code, str, _ in
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
    }
  }

  func switchCameraRoomUuid(_ roomUuid: String?,
                            completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = roomContext.rtcController.switchCamera()
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func joinRtcChannelRoomUuid(_ roomUuid: String?,
                              completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.rtcController.joinRtcChannel { code, str, _ in
      let response = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(response, nil)
    }
  }

  func leaveRtcChannelRoomUuid(_ roomUuid: String?,
                               completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                 -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.rtcController.leaveRtcChannel { code, str, _ in
      let response = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(response, nil)
    }
  }

  func muteMyAudioRoomUuid(_ roomUuid: String?,
                           enableMediaPub: NSNumber?,
                           completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.rtcController.muteMyAudio(enableMediaPub: enableMediaPub != 0) { code, str, _ in
      let response = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(response, nil)
    }
  }

  func unmuteMyAudioRoomUuid(_ roomUuid: String?,
                             enableMediaPub: NSNumber?,
                             completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.rtcController.unmuteMyAudio(enableMediaPub: enableMediaPub != 0) { code, str, _ in
      let response = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(response, nil)
    }
  }

  func muteMyVideoRoomUuid(_ roomUuid: String?,
                           completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.rtcController.muteMyVideo { code, str, _ in
      let response = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(response, nil)
    }
  }

  func unmuteMyVideoRoomUuid(_ roomUuid: String?,
                             completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.rtcController.unmuteMyVideo { code, str, _ in
      let response = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(response, nil)
    }
  }

  func stopScreenShareRoomUuid(_ roomUuid: String?,
                               completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                 -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.rtcController.stopScreenShare { code, str, _ in
      let response = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(response, nil)
    }
  }

  func subscribeRemoteVideoStreamRoomUuid(_ roomUuid: String?,
                                          userUuid: String?,
                                          streamType: NSNumber?,
                                          subscribe: NSNumber?,
                                          completion: @escaping (FLTSimpleResponse?,
                                                                 FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid),
          let userUuid = userUuid,
          let streamType = streamType,
          let subscribe = subscribe else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let type = NEVideoStreamType(rawValue: streamType.uintValue) ?? .high

    if subscribe.boolValue {
      let code = roomContext.rtcController.subscribeRemoteVideoStream(
        userUuid: userUuid,
        streamType: type
      )
      let response = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: code == 0 ? nil : "Failed to subscribe remote video stream.",
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(response, nil)
    } else {
      let code = roomContext.rtcController.unsubscribeRemoteVideoStream(
        userUuid: userUuid,
        streamType: type
      )
      let response = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: code == 0 ? nil : "Failed to unsubscribe remote video stream.",
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(response, nil)
    }
  }

  func subscribeRemoteSubStreamVideoRoomUuid(_ roomUuid: String?,
                                             userUuid: String?,
                                             subscribe: NSNumber?,
                                             completion: @escaping (FLTSimpleResponse?,
                                                                    FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid),
          let subscribe = subscribe,
          let userUuid = userUuid else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    if subscribe.boolValue {
      let code = roomContext.rtcController.subscribeRemoteVideoSubStream(userUuid: userUuid)
      let response = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: code == 0 ? nil : "Failed to subscribe remote subStream video.",
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(response, nil)
    } else {
      let code = roomContext.rtcController.unsubscribeRemoteVideoSubStream(userUuid: userUuid)
      let response = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: code == 0 ? nil : "Failed to unsubscribe remote subStream video.",
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(response, nil)
    }
  }

  func enableVirtualBackgroundRoomUuid(_ roomUuid: String?,
                                       enabled: NSNumber?,
                                       backgroundSource: FLTNERoomVirtualBackgroundSource?,
                                       completion: @escaping (FLTSimpleResponse?, FlutterError?)
                                         -> Void) {
    guard let roomUuid = roomUuid,
          let enabled = enabled,
          let context = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    var source: NERoomVirtualBackgroundSource?
    if let backData = backgroundSource {
      source = NERoomVirtualBackgroundSource()
      source?
        .backgroundSourceType = NERoomVirtualBackgroundSourceType(rawValue: backData
          .backgroundSourceType.uintValue) ?? .color
      source?.color = backData.color.uintValue
      source?.source = backData.source
      source?
        .blurDegree = NERoomBlurDegree(rawValue: backData.blurDegree.uintValue) ?? .high
    }
    let code = context.rtcController.enableVirtualBackground(enable: enabled.boolValue,
                                                             backgroundSource: source)
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func enableAudioAINSRoomUuid(_ roomUuid: String?,
                               enable: NSNumber?,
                               completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let enable = enable,
          let context = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = context.rtcController.enableAudioAINS(enable: enable.boolValue)
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }

  func enableLoopbackRecordingRoomUuid(_ roomUuid: String?,
                                       enable: NSNumber?,
                                       completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let enable = enable,
          let context = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    let code = context.rtcController.enableLoopbackRecording(enable: enable.boolValue)
    let simpleResponse = FLTSimpleResponse.make(
      withCode: NSNumber(value: code),
      msg: nil,
      data1: nil,
      data2: nil,
      data3: nil
    )
    completion(simpleResponse, nil)
  }
}

extension FLTRoomRtcController: FLTAssociativeWrapper {
  func onAttachedToEngine(_ registrar: FlutterPluginRegistrar) {
    FLTRoomRtcControllerApiSetup(registrar.messenger(), self)
  }

  func onDetachedFromEngine(_ registrar: FlutterPluginRegistrar) {
    FLTRoomRtcControllerApiSetup(registrar.messenger(), nil)
  }
}
