// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import Foundation
import NERoomKit
import Flutter
import FileProvider

class FLTRoomSeatController: NSObject, FLTRoomSeatControllerApi {
  func submitSeatRequestRoomUuid(_ roomUuid: String?, seatIndex: NSNumber?, exclusive: NSNumber?, completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let seatIndex = seatIndex,
          let exclusive = exclusive,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.seatController.submitSeatRequest(seatIndex.intValue, exclusive: exclusive.boolValue) { code, str, _ in
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
    }
  }

  func cancelSeatRequestRoomUuid(_ roomUuid: String?, completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.seatController.cancelSeatRequest { code, str, _ in
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
    }
  }

  func leaveSeatRoomUuid(_ roomUuid: String?, completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.seatController.leaveSeat { code, str, _ in
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
    }
  }

  func approveSeatRequestRoomUuid(_ roomUuid: String?, user: String?, completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let user = user,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.seatController.approveSeatRequest(user, callback: { code, str, _ in
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
    })
  }

  func rejectSeatRequestRoomUuid(_ roomUuid: String?, user: String?, completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let user = user,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.seatController.rejectSeatRequest(user, callback: { code, str, _ in
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
    })
  }

  func kickSeatRoomUuid(_ roomUuid: String?, user: String?, completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let user = user,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.seatController.kickSeat(user, callback: { code, str, _ in
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
    })
  }

  func sendSeatInvitationRoomUuid(_ roomUuid: String?, seatIndex: NSNumber?, user: String?, completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let seatIndex = seatIndex,
          let user = user,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.seatController.sendSeatInvitation(seatIndex.intValue, userUuid: user, callback: { code, str, _ in
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
    })
  }

  func cancelSeatInvitationRoomUuid(_ roomUuid: String?, user: String?, completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let user = user,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.seatController.cancelSeatInvitation(user, callback: { code, str, _ in
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
    })
  }

  func acceptSeatInvitationRoomUuid(_ roomUuid: String?, completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.seatController.acceptSeatInvitation { code, str, _ in
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
    }
  }

  func rejectSeatInvitationRoomUuid(_ roomUuid: String?, completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.seatController.rejectSeatInvitation { code, str, _ in
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
    }
  }

  func closeSeatsRoomUuid(_ roomUuid: String?, seatIndices: [NSNumber]?, completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let seatIndices = seatIndices,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.seatController.closeSeat(seatIndices.map { number in
      number.intValue
    }) { code, str, _ in
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
    }
  }

  func openSeatsRoomUuid(_ roomUuid: String?, seatIndices: [NSNumber]?, completion: @escaping (FLTSimpleResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let seatIndices = seatIndices,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: -1),
        msg: paramsError,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
      return
    }
    roomContext.seatController.openSeat(seatIndices.map { number in
      number.intValue
    }) { code, str, _ in
      let simpleResponse = FLTSimpleResponse.make(
        withCode: NSNumber(value: code),
        msg: str,
        data1: nil,
        data2: nil,
        data3: nil
      )
      completion(simpleResponse, nil)
    }
  }

  func getSeatInfoRoomUuid(_ roomUuid: String?, completion: @escaping (FLTSeatInfoResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSeatInfoResponse.make(withCode: NSNumber(value: -1), msg: paramsError, seatInfo: nil)
      completion(simpleResponse, nil)
      return
    }
    roomContext.seatController.getSeatInfo { code, str, seatInfo in
      let simpleResponse = FLTSeatInfoResponse.make(withCode: NSNumber(value: code), msg: str, seatInfo: FLTSeatInfo.make(withCreator: seatInfo?.creator, managers: seatInfo?.managers, seatItems: seatInfo?.seatItems.map { seatItem in
        FLTSeatItem.make(withIndex: NSNumber(value: seatItem.index), status: NSNumber(value: seatItem.status.rawValue), user: seatItem.user, userName: seatItem.userName, icon: seatItem.icon, onSeatType: NSNumber(value: seatItem.onSeatType.rawValue), updated: NSNumber(value: seatItem.updated))
      }))
      completion(simpleResponse, nil)
    }
  }

  func getSeatRequestListRoomUuid(_ roomUuid: String?, completion: @escaping (FLTSeatRequestListResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSeatRequestListResponse.make(withCode: NSNumber(value: -1), msg: paramsError, seatRequestList: [])
      completion(simpleResponse, nil)
      return
    }
    roomContext.seatController.getSeatRequestList { code, str, requestItems in
      guard let requestItems = requestItems else {
        let simpleResponse = FLTSeatRequestListResponse.make(withCode: NSNumber(value: code), msg: str, seatRequestList: [])
        completion(simpleResponse, nil)
        return
      }

      let simpleResponse = FLTSeatRequestListResponse.make(withCode: NSNumber(value: code), msg: str, seatRequestList: requestItems.map { requestItem in
        FLTSeatRequestItem.make(withIndex: NSNumber(value: requestItem.index), user: requestItem.user, userName: requestItem.userName, icon: requestItem.icon)
      })
      completion(simpleResponse, nil)
    }
  }

  func getSeatInvitationListRoomUuid(_ roomUuid: String?, completion: @escaping (FLTSeatInvitationListResponse?, FlutterError?) -> Void) {
    guard let roomUuid = roomUuid,
          let roomContext = NERoomKit.shared().roomService.getRoomContext(roomUuid: roomUuid)
    else {
      let simpleResponse = FLTSeatInvitationListResponse.make(withCode: NSNumber(value: -1), msg: paramsError, seatInvitationList: [])
      completion(simpleResponse, nil)
      return
    }
    roomContext.seatController.getSeatInvitationList { code, str, seatInvitationItems in
      guard let seatInvitationItems = seatInvitationItems else {
        let simpleResponse = FLTSeatInvitationListResponse.make(withCode: NSNumber(value: code), msg: str, seatInvitationList: [])
        completion(simpleResponse, nil)
        return
      }
      let simpleResponse = FLTSeatInvitationListResponse.make(withCode: NSNumber(value: code), msg: str, seatInvitationList: seatInvitationItems.map { seatInvitationItem in
        FLTSeatInvitationItem.make(withIndex: NSNumber(value: seatInvitationItem.index), user: seatInvitationItem.user)
      })
      completion(simpleResponse, nil)
    }
  }
}

extension FLTRoomSeatController: FLTAssociativeWrapper {
  func onAttachedToEngine(_ registrar: FlutterPluginRegistrar) {
    FLTRoomSeatControllerApiSetup(registrar.messenger(), self)
  }

  func onDetachedFromEngine(_ registrar: FlutterPluginRegistrar) {
    FLTRoomSeatControllerApiSetup(registrar.messenger(), nil)
  }
}
