// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of netease_roomkit;

///
/// 消息通道服务
///
@HawkEntryPoint()
abstract class NEMessageChannelService {
  ///
  /// 添加IM自定义透传消息监听
  /// [callback] 消息监听器
  ///
  void addMessageChannelCallback(NEMessageChannelCallback callback);

  ///
  /// 移除IM自定义透传消息监听
  /// [callback] 消息监听器
  ///
  void removeMessageChannelCallback(NEMessageChannelCallback callback);

  ///
  /// 给房间内的[userUuid]发送透传消息，如房间内信令。
  /// 如果需要发送聊天消息请使用[NERoomChatController]。
  /// [roomUuid] 房间ID
  /// [userUuid] 用户ID
  /// [data] 自定义的消息内容
  /// [crossAppAuthorization] 跨房间发消息所需的鉴权信息
  ///
  Future<VoidResult> sendCustomMessage(
      String roomUuid, String userUuid, int commandId, String data,
      {NECrossAppAuthorization? crossAppAuthorization});
}

/// IM自定义透传消息回调
/// [throughMessage] 透传消息
typedef ReceiveCustomMessageCallback = void Function(
    NECustomMessage throughMessage);

class NEMessageChannelCallback {
  final ReceiveCustomMessageCallback? onReceiveCustomMessage;
  NEMessageChannelCallback({this.onReceiveCustomMessage});
}

class NECustomMessage {
  final String? roomUuid;
  final String? senderUuid;
  final int commandId;
  final String data;
  NECustomMessage(this.roomUuid, this.senderUuid, this.commandId, this.data);
}
