// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of netease_roomkit;

///
/// RTC 用户角色
///
class NERoomRtcClientRole {
  ///
  /// 主播
  ///
  static const int BROADCASTER = 0;

  ///
  /// 观众
  ///
  static const int AUDIENCE = 1;
}

///
/// 通信场景
///
class NERoomRtcChannelProfile {
  ///
  /// （默认）通信场景。该场景下，房间内所有用户都可以发布和接收音、视频流。适用于语音通话、视频群聊等应用场景。
  ///
  static const int communication = 0;

  ///
  ///  直播场景。该场景有主播和观众两种用户角色，可以通过setClientRole设置。主播可以发布和接收音视频流，观众直接接收流。
  ///  适用于语聊房、视频直播、互动大班课等应用场景。
  ///
  static const int liveBroadcasting = 1;
}

class NERoomRtcAudioProfile {
  ///
  /// 默认设置。语音场景下为 STANDARD，音乐场景下为 HIGH_QUALITY。
  ///
  static const int DEFAULT = 0;

  ///
  /// 标准音质模式。采样率为 16 kHz、语音编码、单声道、编码码率最大值为 20 Kbps。
  ///
  static const int STANDARD = 1;

  ///
  /// 标准扩展模式。采样率为 16 kHz、语音编码、单声道、编码码率最大值为 32 Kbps。
  ///
  static const int STANDARD_EXTEND = 2;

  ///
  /// 中等音质模式。采样率为 48 kHz、音乐编码、单声道、编码码率最大值为 32 Kbps。
  ///
  static const int MIDDLE_QUALITY = 3;

  ///
  /// 中等音质模式（立体音）。采样率为 48 kHz、音乐编码、双声道、编码码率最大值为 64 Kbps。
  ///
  static const int MIDDLE_QUALITY_STEREO = 4;

  ///
  /// 高音质模式。采样率为 48 kHz、音乐编码、单声道、编码码率最大值为 64 Kbps。
  ///
  static const int HIGH_QUALITY = 5;

  ///
  /// 高音质模式（立体音）。采样率为 48 kHz、音乐编码、双声道、编码码率最大值为 128 Kbps。
  ///
  static const int HIGH_QUALITY_STEREO = 6;
}

class NERoomRtcAudioScenario {
  ///
  /// 默认设置为Speech。
  ///
  static const int DEFAULT = 0;

  ///
  /// 语音场景. AudioProfile 推荐使用 MIDDLE_QUALITY 及以下。
  ///
  static const int SPEECH = 1;

  ///
  /// 音乐场景。AudioProfile 推荐使用 MIDDLE_QUALITY_STEREO 及以上。
  ///
  static const int MUSIC = 2;

  ///
  /// 语聊房场景，AudioProfile推荐 HIGH_QUALITY及以上。
  ///
  static const int CHATROOM = 3;
}

///
/// 房间RTC媒体发布类型
///
class NERoomRtcMediaPublishType {
  /// 音频。
  static const audio = 0;
}
