// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of netease_roomkit;

@HawkEntryPoint()
abstract class NERoomChatController extends NERoomBaseController {
  ///加入聊天室
  Future<VoidResult> joinChatroom();

  ///离开聊天室
  Future<VoidResult> leaveChatroom();

  ///发送广播消息
  ///[message] 文本消息
  Future<VoidResult> sendBroadcastTextMessage(String message);

  ///发送定向消息
  ///[userUuid] 对方用户id
  ///[message] 文本消息
  Future<VoidResult> sendDirectTextMessage(String userUuid, String message);

  ///给多人发送消息
  ///[userUuids] 用户id列表
  ///[message] 文本消息
  Future<VoidResult> sendGroupTextMessage(
      List<String> userUuids, String message);

  ///获取聊天室成员
  ///[queryType] 成员查询类型
  ///[limit] 人数限制
  Future<NEResult<List<NEChatroomMember>>> fetchChatroomMembers(
      NEChatroomMemberQueryType queryType, int limit);

  ///
  /// 发送图片消息
  /// * [messageUuid] 消息标识，可用于监听附件上传的进度，可通过 [UUID.randomUUID()] 生成
  /// * [imagePath] 图片路径
  /// * [userUuids] 用户id列表，为空表示全员消息
  ///
  Future<VoidResult> sendImageMessage(
    String messageUuid,
    String imagePath,
    List<String>? userUuids,
  );

  ///
  /// 发送文件消息
  /// * [messageUuid] 消息标识，可用于监听附件上传的进度，可通过 [UUID.randomUUID()] 生成
  /// * [filePath] 文件路径
  /// * [userUuids] 用户id列表，为空表示全员消息
  ///
  Future<VoidResult> sendFileMessage(
    String messageUuid,
    String filePath,
    List<String>? userUuids,
  );

  ///
  /// 下载文件消息附件。
  /// * [messageUuid] 消息唯一ID。
  ///
  Future<VoidResult> downloadAttachment(
    String messageUuid,
  );

  ///
  /// 取消下载文件消息附件。
  /// * [messageUuid] 消息唯一ID。
  ///
  Future<VoidResult> cancelDownloadAttachment(
    String messageUuid,
  );
}
