// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.
part of netease_roomkit;

///
/// 房间成员
///
abstract class NERoomMember with AttachmentMixin {
  ///
  /// 用户id
  ///
  String get uuid;

  ///
  /// 用户名
  ///
  String get name;

  ///
  /// 用户头像 URL 地址
  ///
  String? get avatar;

  ///
  /// 角色
  ///
  NERoomRole get role;

  ///
  /// 音频是否打开
  ///
  bool get isAudioOn;

  ///
  /// 视频是否打开
  ///
  bool get isVideoOn;

  ///
  /// 当前成员是否在RTC房间内
  ///
  bool get isInRtcChannel;

  ///
  /// 当前成员是否在聊天室内
  ///
  bool get isInChatroom;

  ///
  /// 当前成员是否正在屏幕共享中
  ///
  bool get isSharingScreen;

  ///
  /// 当前成员是否在共享白板
  ///
  bool get isSharingWhiteboard;

  ///
  /// 属性
  ///
  Map<String, String> get properties;

  ///
  /// 当前应用属性 [NEClientType]
  ///
  NEClientType get clientType;

  Map toJson() => {
        'uuid': uuid,
        'name': name,
        'avatar': avatar,
        'role': role.toJson(),
        'isAudioOn': isAudioOn,
        'isVideoOn': isVideoOn,
        'isInRtcChannel': isInRtcChannel,
        'isInChatroom': isInChatroom,
        'isSharingScreen': isSharingScreen,
        'isSharingWhiteboard': isSharingWhiteboard,
        'clientType': clientType,
        'properties': properties,
      };
}

///
/// 用户的终端类型
///
enum NEClientType {
  ///
  /// 未知平台
  ///
  unknown,

  ///
  /// iOS平台
  ///
  ios,

  ///
  /// Android平台
  ///
  android,

  ///
  /// windows平台
  ///
  pc,

  ///
  /// web平台
  ///
  web,

  ///
  /// sip 平台
  ///
  sip,

  ///
  /// mac平台
  ///
  mac,

  ///
  /// 小程序平台
  ///
  miniapp,
}

extension NEClientTypeExtension on NEClientType {
  static NEClientType fromType(String type) {
    return NEClientType.values.firstWhere(
      (element) => element.name == type.toLowerCase(),
      orElse: () => NEClientType.unknown,
    );
  }
}
