// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of netease_roomkit;

/// 房间服务
@HawkEntryPoint()
abstract class NERoomService {
  ///
  /// 创建一个房间，不会加入房间
  /// [params]   创建房间的参数
  /// [options]  创建房间的选项
  ///
  Future<VoidResult> createRoom(
      NECreateRoomParams params, NECreateRoomOptions options);

  ///
  /// 加入一个房间。成功加入房间后会返回一个 [NERoomContext] 实例，可以进行房间内相关操作。
  /// [params]   加入房间的参数
  /// [options]  加入房间的选项
  ///
  Future<NEResult<NERoomContext>> joinRoom(
      NEJoinRoomParams params, NEJoinRoomOptions options);

  ///
  /// 加入房间前预览。成功预览房间后会返回一个 [NEPreviewRoomContext] 实例，可以进行预览房间相关操作。
  /// [params]   预览房间的参数
  /// [options]  预览房间的选项
  ///
  Future<NEResult<NEPreviewRoomContext>> previewRoom(
      NEPreviewRoomParams params, NEPreviewRoomOptions options);

  ///
  /// 根据 [roomUuid] 获取 [RoomContext] 实例
  /// [roomUuid] 房间id
  ///
  NERoomContext? getRoomContext(String roomUuid);
}
