// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of netease_roomkit;

/// 白板控制器
@HawkEntryPoint()
abstract class NERoomWhiteboardController extends NERoomBaseController {
  ///开始白板共享
  Future<VoidResult> startWhiteboardShare();

  /// 停止白板共享
  Future<VoidResult> stopWhiteboardShare();

  Future<VoidResult> stopMemberWhiteboardShare(String userUuid);

  ///查询白板共享者的userUuid
  String? getWhiteboardSharingUserUuid();

  /// 白板Widget
  @HawkApi(ignore: true)
  Widget createWhiteboardView({
    Set<Factory<OneSequenceGestureRecognizer>>? gestureRecognizers,
    Color? backgroundColor,
  });

  ///查询当前用户是否正在共享白板
  bool isSharingWhiteboard();

  ///查询用户是否正在共享白板
  bool isWhiteboardSharing(String userId);

  /// 设置显示白板工具栏
  Future<NEResult<void>> showWhiteboardTools(bool show);

  ///
  /// 设置白板背景颜色，只对当前客户端有影响
  /// [color] 白板背景颜色，如"rgb(255, 255, 255)"，"rgba(255, 255, 255, 0)"
  ///
  Future<VoidResult> setCanvasBackgroundColor(String color);

  ///
  /// 将白板和被标注物的坐标绑定
  /// [width] 被标注物的宽度(一般为视频画面宽度)
  /// [height] 被标注物的高度(一般为视频画面高度度)
  ///
  Future<VoidResult> lockCameraWithContent(int width, int height);
}
