// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of netease_roomkit;

///
/// RoomKit 入口类
///
@HawkEntryPoint()
abstract class NERoomKit {
  static final _instance = _NERoomKitImpl();

  /// 获取全局唯一的 RoomKit 实例
  static NERoomKit get instance => _instance;

  ///
  /// 初始化
  /// [options] 初始化参数
  ///
  Future<VoidResult> initialize(NERoomKitOptions options);

  ///
  /// 切换语言
  ///
  Future<VoidResult> switchLanguage(NERoomLanguage? language);

  ///
  /// 查询初始化状态
  ///
  bool get isInitialized;

  ///
  /// 获取版本号信息
  ///
  Future<NESDKVersions> get sdkVersions;

  ///
  /// 获取鉴权服务，提供登录、登出能力，返回 [NEAuthService] 实例。
  ///
  NEAuthService get authService;

  ///
  /// 获取房间服务，提供创建、加入房间能力，返回 [NERoomService] 实例。
  ///
  NERoomService get roomService;

  ///
  /// 获取消息通道服务，提供房间内发送自定义消息能力，返回[NEMessageChannelService] 实例。
  ///
  NEMessageChannelService get messageChannelService;
}

///
/// 版本信息
/// [imVersion] IM版本
/// [rtcVersion] rtc版本
/// [whiteboardVersion] 白板版本
/// [roomKitVersion] platform roomKit版本
/// [fltRoomKitVersion] flutter roomKit版本
///
class NESDKVersions {
  final String imVersion;
  final String rtcVersion;
  final String whiteboardVersion;
  final String roomKitVersion;
  final String fltRoomKitVersion;

  NESDKVersions({
    required this.imVersion,
    required this.rtcVersion,
    required this.whiteboardVersion,
    required this.roomKitVersion,
    required this.fltRoomKitVersion,
  });
}

///
/// 组件支持的语言类型
///
class NERoomLanguage {
  static const automatic = NERoomLanguage('*');
  static const chinese = NERoomLanguage('zh');
  static const english = NERoomLanguage('en');
  static const japanese = NERoomLanguage('ja');

  final String languageCode;

  const NERoomLanguage(this.languageCode);

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is NERoomLanguage &&
          runtimeType == other.runtimeType &&
          languageCode == other.languageCode;

  @override
  int get hashCode => languageCode.hashCode;
}
