// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of netease_roomkit;

/// 视频画布创建工厂
class NERtcVideoRendererFactory {
  /// 创建 [NERtcVideoRenderer], 同时进行初始化
  static Future<NERtcVideoRenderer> createVideoRenderer(String roomUuid,
      {NERtcVideoRendererEventListener? listener}) async {
    NERtcVideoRenderer renderer =
        _NERtcVideoRendererImpl(roomUuid, listener: listener);
    await renderer.initialize();
    return renderer;
  }
}

/// 视频渲染事件监听器
abstract class NERtcVideoRendererEventListener {
  void onFrameResolutionChanged(int width, int height, int rotation);

  void onFirstFrameRendered();
}

/// 视频渲染画布
abstract class NERtcVideoRenderer extends ValueNotifier<_NERtcVideoValue> {
  NERtcVideoRenderer({this.rendererEventLister})
      : super(const _NERtcVideoValue.uninitialized());

  NERtcVideoRendererEventListener? rendererEventLister;

  int? get textureId;

  bool get canRender;

  Future<bool> initialize();

  /// 添加为远端用户视频画布
  Future<int> attachToRemoteVideo(String userUuid);

  /// 添加为远端用户辅流视频画布
  Future<int> attachToRemoteSubStreamVideo(String userUuid);

  /// 添加为本地用户视频画布
  Future<int> attachToLocalVideo();

  /// 添加为本地用户辅流视频画布
  Future<int> attachToLocalSubStreamVideo();

  /// 取消设置画布
  Future<int> detach();

  /// 设置画面是否镜像
  Future<int> setMirror(bool mirror);

  /// 获取画面是否镜像
  bool isMirror();

  @override
  @mustCallSuper
  Future<int> dispose() async {
    super.dispose();
    return 0;
  }
}
