// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of netease_roomkit;

class _NEMessageServiceImpl extends NEMessageChannelService
    with MessageEventSink {
  final _platform = MessageChannelServicePlatform.instance;
  final _messageListeners = <NEMessageChannelCallback>{};

  _NEMessageServiceImpl() {
    _platform.addMessageListener(this);
  }

  @override
  void addMessageChannelCallback(NEMessageChannelCallback callback) {
    _messageListeners.add(callback);
  }

  @override
  void removeMessageChannelCallback(NEMessageChannelCallback callback) {
    _messageListeners.remove(callback);
  }

  @override
  void onReceivePassThroughMessage(PassThroughMessageEvent evt) {
    for (var element in _messageListeners) {
      element.onReceiveCustomMessage?.call(NECustomMessage(
        evt.roomUuid,
        evt.senderUuid,
        evt.commandId,
        evt.data,
      ));
    }
  }

  @override
  Future<VoidResult> sendCustomMessage(
      String roomUuid, String userUuid, int commandId, String data,
      {NECrossAppAuthorization? crossAppAuthorization}) {
    return _platform.sendCustomMessage(roomUuid, userUuid, commandId, data,
        crossAppAuthorization: crossAppAuthorization);
  }
}
