// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of netease_roomkit;

class _NEPreviewRoomContextImpl extends NEPreviewRoomContext
    with _AloggerMixin, PreviewRoomEventSink {
  final _eventCallbacks = <NEPreviewRoomEventCallback>{};

  _NEPreviewRoomContextImpl() {
    final _platform = PreviewRoomRtcControllerPlatform.instance;
    _platform.registerEventListener(this);
  }

  @override
  NEPreviewRoomRtcController get previewController =>
      _NEPreviewRoomRtcControllerImpl();

  @override
  void onRtcVirtualBackgroundSourceEnabled(
      RtcVirtualBackgroundSourceEvent event) {
    final enabled = event.enabled;
    final reason = event.reason;
    commonLogger.i(
        'preview onRtcVirtualBackgroundSourceEnabled: enabled:$enabled,reason:$reason');
    for (var callback in _eventCallbacks.copy()) {
      callback.rtcVirtualBackgroundSourceEnabled?.call(enabled, reason);
    }
  }

  @override
  void addEventCallback(NEPreviewRoomEventCallback callback) {
    _eventCallbacks.add(callback);
  }

  @override
  void removeEventCallback(NEPreviewRoomEventCallback callback) {
    _eventCallbacks.remove(callback);
  }
}
