// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of netease_roomkit;

class _NERoomChatControllerImpl extends NERoomChatController
    with _AloggerMixin {
  final _platform = RoomChatControllerPlatform.instance;
  final String roomUuid;

  @override
  final bool isSupported;

  _NERoomChatControllerImpl(this.isSupported, this.roomUuid);

  @override
  Future<VoidResult> joinChatroom() {
    apiLogger.i('joinChatroom:');
    return _platform.joinChatroom(roomUuid);
  }

  @override
  Future<VoidResult> leaveChatroom() {
    apiLogger.i('leaveChatroom:');
    return _platform.leaveChatroom(roomUuid);
  }

  @override
  Future<VoidResult> sendBroadcastTextMessage(String message) {
    apiLogger.i('sendBroadcastTextMessage:roomUuid:message:$message');
    return _platform.sendBroadcastTextMessage(roomUuid, message);
  }

  @override
  Future<VoidResult> sendDirectTextMessage(String userUuid, String message) {
    apiLogger.i('sendDirectTextMessage: userUuid:$userUuid,message:$message');
    return _platform.sendDirectTextMessage(roomUuid, userUuid, message);
  }

  @override
  Future<VoidResult> sendGroupTextMessage(
      List<String> userUuids, String message) {
    apiLogger.i('sendGroupTextMessage:userUuids:$userUuids,message:$message');
    return _platform.sendGroupTextMessage(roomUuid, userUuids, message);
  }

  @override
  Future<NEResult<List<NEChatroomMember>>> fetchChatroomMembers(
      NEChatroomMemberQueryType queryType, int limit) {
    apiLogger.i('fetchChatRoomMembers:queryType:$queryType, limit:$limit');
    return _platform.fetchChatRoomMembers(roomUuid, queryType, limit);
  }

  @override
  Future<VoidResult> cancelDownloadAttachment(String messageUuid) {
    apiLogger.i('cancelDownloadAttachment:messageUuid:$messageUuid');
    return _platform.cancelDownloadAttachment(roomUuid, messageUuid);
  }

  @override
  Future<VoidResult> downloadAttachment(String messageUuid) {
    apiLogger.i('downloadAttachment:messageUuid:$messageUuid');
    return _platform.downloadAttachment(roomUuid, messageUuid);
  }

  @override
  Future<VoidResult> sendFileMessage(
      String messageUuid, String filePath, List<String>? userUuids) {
    apiLogger.i(
        'sendFileMessage:messageUuid:$messageUuid,filePath:$filePath,userUuids:$userUuids');
    return _platform.sendFileMessage(
        roomUuid, messageUuid, filePath, userUuids);
  }

  @override
  Future<VoidResult> sendImageMessage(
      String messageUuid, String imagePath, List<String>? userUuids) {
    apiLogger.i(
        'sendImageMessage:messageUuid:$messageUuid,imagePath:$imagePath,userUuids:$userUuids');
    return _platform.sendImageMessage(
        roomUuid, messageUuid, imagePath, userUuids);
  }
}
