// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of netease_roomkit;

class _NERoomRtcControllerImpl extends NERoomRtcController with _AloggerMixin {
  final _platform = RoomRtcControllerPlatform.instance;
  final _NERoomContextImpl roomContext;
  final String roomUuid;

  @override
  final bool isSupported;

  _NERoomRtcControllerImpl(this.isSupported, this.roomContext)
      : roomUuid = roomContext.roomUuid;

  @override
  Future<VoidResult> joinRtcChannel() {
    apiLogger.i('joinRtcChannel');
    return _platform.joinRtcChannel(roomUuid);
  }

  @override
  Future<VoidResult> leaveRtcChannel() {
    apiLogger.i('leaveRtcChannel');
    return _platform.leaveRtcChannel(roomUuid);
  }

  @override
  Future<VoidResult> muteMyAudio([bool enableMediaPub = false]) {
    apiLogger.i('muteMyAudio: enableMediaPub=$enableMediaPub');
    return _platform.muteMyAudio(roomUuid, enableMediaPub);
  }

  @override
  Future<VoidResult> unmuteMyAudio([bool enableMediaPub = true]) {
    apiLogger.i('unmuteMyAudio: enableMediaPub=$enableMediaPub');
    return _platform.unmuteMyAudio(roomUuid, enableMediaPub);
  }

  @override
  Future<VoidResult> enableMediaPub(int mediaType, bool enable) {
    apiLogger.i('enableMediaPub: mediaType=$mediaType, enable=$enable');
    return _platform.enableMediaPub(roomUuid, mediaType, enable);
  }

  @override
  Future<VoidResult> pauseLocalAudioRecording() {
    apiLogger.i('pauseLocalAudioRecording');
    return _platform.pauseLocalAudioRecording(roomUuid);
  }

  @override
  Future<VoidResult> resumeLocalAudioRecording() {
    apiLogger.i('resumeLocalAudioRecording');
    return _platform.resumeLocalAudioRecording(roomUuid);
  }

  @override
  Future<VoidResult> muteMyVideo() {
    apiLogger.i('muteMyVideo');
    return _platform.muteMyVideo(roomUuid);
  }

  @override
  Future<VoidResult> unmuteMyVideo() {
    apiLogger.i('unmuteMyVideo');
    return _platform.unmuteMyVideo(roomUuid);
  }

  @override
  Future<VoidResult> pauseLocalVideoCapture() {
    apiLogger.i('pauseLocalVideoCapture');
    return _platform.pauseLocalVideoCapture(roomUuid);
  }

  @override
  Future<VoidResult> resumeLocalVideoCapture() {
    apiLogger.i('resumeLocalVideoCapture');
    return _platform.resumeLocalVideoCapture(roomUuid);
  }

  @override
  Future<VoidResult> stopScreenShare() {
    apiLogger.i('stopScreenShare');
    return _platform.stopScreenShare(roomUuid);
  }

  @override
  Future<VoidResult> stopMemberScreenShare(String uuid) {
    apiLogger.i('stopMemberScreenShare: uuid=$uuid');
    return _platform.stopMemberScreenShare(roomUuid, uuid);
  }

  @override
  Future<VoidResult> subscribeRemoteVideoStream(
    String userUuid,
    NEVideoStreamType streamType,
  ) {
    apiLogger.i(
        'subscribeRemoteVideoStream: userUuid=$userUuid, streamType=$streamType');
    return _platform.subscribeRemoteVideoStream(
        roomUuid, userUuid, streamType, true);
  }

  @override
  Future<VoidResult> unsubscribeRemoteVideoStream(
    String userUuid,
    NEVideoStreamType streamType,
  ) {
    apiLogger.i(
        'unsubscribeRemoteVideoStream: userUuid=$userUuid, streamType=$streamType');
    return _platform.subscribeRemoteVideoStream(
        roomUuid, userUuid, streamType, false);
  }

  @override
  Future<VoidResult> adjustUserPlaybackSignalVolume(
      String userUuid, int volume) {
    apiLogger.i(
        'adjustUserPlaybackSignalVolume: userUuid=$userUuid, volume=$volume');
    return _platform.adjustUserPlaybackSignalVolume(roomUuid, userUuid, volume);
  }

  @override
  Future<VoidResult> adjustPlaybackSignalVolume(int volume) {
    apiLogger.i('adjustPlaybackSignalVolume: volume=$volume');
    return _platform.adjustPlaybackSignalVolume(roomUuid, volume);
  }

  @override
  Future<VoidResult> subscribeRemoteVideoSubStream(String userUuid) {
    apiLogger.i('subscribeRemoteVideoSubStream: userUuid=$userUuid');
    return _platform.subscribeRemoteSubStreamVideo(roomUuid, userUuid, true);
  }

  @override
  Future<VoidResult> unsubscribeRemoteVideoSubStream(String userUuid) {
    apiLogger.i('unsubscribeRemoteVideoSubStream: userUuid=$userUuid');
    return _platform.subscribeRemoteSubStreamVideo(roomUuid, userUuid, false);
  }

  @override
  Future<VoidResult> startScreenShare({String? iosAppGroup}) {
    apiLogger.i('startScreenShare: iosAppGroup=$iosAppGroup');
    return _platform.startScreenShare(roomUuid, iosAppGroup: iosAppGroup);
  }

  @override
  String? getScreenSharingUserUuid() {
    final uuid = roomContext.screenSharingMember?.uuid;
    return roomContext.getMember(uuid) != null ? uuid : null;
  }

  @override
  Future<VoidResult> muteMemberAudio(String uuid) {
    apiLogger.i('muteMemberAudio: uuid=$uuid');
    return _platform.muteMemberAudio(roomUuid, uuid);
  }

  @override
  Future<VoidResult> muteMemberVideo(String uuid) {
    apiLogger.i('muteMemberVideo: uuid=$uuid');
    return _platform.muteMemberVideo(roomUuid, uuid);
  }

  @override
  Future<VoidResult> unmuteMemberAudio(String uuid) {
    apiLogger.i('unmuteMemberAudio: uuid=$uuid');
    return _platform.unmuteMemberAudio(roomUuid, uuid);
  }

  @override
  Future<VoidResult> unmuteMemberVideo(String uuid) {
    apiLogger.i('unmuteMemberVideo: uuid=$uuid');
    return _platform.unmuteMemberVideo(roomUuid, uuid);
  }

  @override
  Future<VoidResult> enableAudioVolumeIndication(bool enable, int interval,
      [bool vad = false]) {
    apiLogger.i(
        'enableAudioVolumeIndication: enable=$enable, interval=$interval,vad=$vad');
    return _platform.enableAudioVolumeIndication(
        roomUuid, enable, interval, vad);
  }

  @override
  Future<bool> isSpeakerphoneOn() {
    return _platform.isSpeakerphoneOn(roomUuid);
  }

  @override
  Future<VoidResult> setSpeakerphoneOn(bool on) {
    apiLogger.i('setSpeakerphoneOn: on=$on');
    return _platform.setSpeakerphoneOn(roomUuid, on);
  }

  @override
  Future<VoidResult> startAudioDump(NEAudioDumpType type) {
    apiLogger.i('startAudioDump: type=$type');
    return _platform.startAudioDump(roomUuid, type);
  }

  @override
  Future<VoidResult> stopAudioDump() {
    apiLogger.i('stopAudioDump');
    return _platform.stopAudioDump(roomUuid);
  }

  @override
  Future<VoidResult> startRtcChannelMediaRelay() {
    apiLogger.i('startRtcChannelMediaRelay');
    return _platform.startRtcChannelMediaRelay(roomUuid);
  }

  @override
  Future<VoidResult> stopRtcChannelMediaRelay() {
    apiLogger.i('stopRtcChannelMediaRelay');
    return _platform.stopRtcChannelMediaRelay(roomUuid);
  }

  @override
  Future<VoidResult> startAudioMixing(NECreateAudioMixingOption option) {
    apiLogger.i('startAudioMixing');
    return _platform.startAudioMixing(roomUuid, option);
  }

  @override
  Future<VoidResult> playEffect(int id, NECreateAudioEffectOption option) {
    apiLogger.i('playEffect id=$id');
    return _platform.playEffect(roomUuid, id, option);
  }

  @override
  Future<VoidResult> stopAudioMixing() {
    apiLogger.i('stopAudioMixing');
    return _platform.stopAudioMixing(roomUuid);
  }

  @override
  Future<VoidResult> stopEffect(int id) {
    apiLogger.i('stopEffect id=$id');
    return _platform.stopEffect(roomUuid, id);
  }

  @override
  Future<VoidResult> setAudioMixingSendVolume(int volume) {
    return _platform.setAudioMixingSendVolume(roomUuid, volume);
  }

  @override
  Future<VoidResult> setAudioMixingPlaybackVolume(int volume) {
    return _platform.setAudioMixingPlaybackVolume(roomUuid, volume);
  }

  @override
  Future<VoidResult> setEffectSendVolume(int id, int volume) {
    return _platform.setEffectSendVolume(roomUuid, id, volume);
  }

  @override
  Future<VoidResult> setEffectPlaybackVolume(int id, int volume) {
    return _platform.setEffectPlaybackVolume(roomUuid, id, volume);
  }

  @override
  Future<VoidResult> stopAllEffects() {
    apiLogger.i('stopAllEffects');
    return _platform.stopAllEffects(roomUuid);
  }

  @override
  Future<VoidResult> enableEarBack(int volume) {
    apiLogger.i('enableEarBack');
    return _platform.enableEarBack(roomUuid, true, volume);
  }

  @override
  Future<VoidResult> disableEarBack() {
    apiLogger.i('disableEarBack');
    return _platform.enableEarBack(roomUuid, false, 0);
  }

  @override
  Future<VoidResult> switchCamera() {
    apiLogger.i('switchCamera');
    return _platform.switchCamera(roomUuid);
  }

  @override
  Future<VoidResult> addBeautyFilter(String path) {
    apiLogger.i('addBeautyFilter: path=$path');
    return _platform.addBeautyFilter(roomUuid, path);
  }

  @override
  Future<VoidResult> addBeautySticker(String path) {
    apiLogger.i('addBeautySticker: path=$path');
    return _platform.addBeautySticker(roomUuid, path);
  }

  @override
  Future<VoidResult> enableBeauty(bool isOpenBeauty) {
    apiLogger.i('enableBeauty: isOpenBeauty=$isOpenBeauty');
    return _platform.enableBeauty(roomUuid, isOpenBeauty);
  }

  @override
  Future<VoidResult> removeBeautyFilter() {
    apiLogger.i('removeBeautyFilter');
    return _platform.removeBeautyFilter(
      roomUuid,
    );
  }

  @override
  Future<VoidResult> removeBeautySticker() {
    apiLogger.i('removeBeautySticker');
    return _platform.removeBeautySticker(
      roomUuid,
    );
  }

  @override
  Future<VoidResult> setBeautyEffect(
      NERoomBeautyEffectType beautyType, double level) {
    return _platform.setBeautyEffect(roomUuid, beautyType, level);
  }

  @override
  Future<VoidResult> setBeautyFilterLevel(double level) {
    return _platform.setBeautyFilterLevel(roomUuid, level);
  }

  @override
  Future<VoidResult> startBeauty() {
    apiLogger.i('startBeauty');
    return _platform.startBeauty(
      roomUuid,
    );
  }

  @override
  Future<VoidResult> stopBeauty() {
    apiLogger.i('stopBeauty');
    return _platform.stopBeauty(
      roomUuid,
    );
  }

  @override
  Future<VoidResult> enableVirtualBackground(
      bool enabled, NERoomVirtualBackgroundSource backgroundSource) {
    return _platform.enableVirtualBackground(
        roomUuid, enabled, backgroundSource);
  }

  @override
  Future<VoidResult> adjustRecordingSignalVolume(int volume) {
    apiLogger.i('adjustRecordingSignalVolume');
    return _platform.adjustRecordingSignalVolume(roomUuid, volume);
  }

  @override
  Future<VoidResult> pauseAudioMixing() {
    apiLogger.i('pauseAudioMixing');
    return _platform.pauseAudioMixing(roomUuid);
  }

  @override
  Future<VoidResult> resumeAudioMixing() {
    apiLogger.i('resumeAudioMixing');
    return _platform.resumeAudioMixing(roomUuid);
  }

  @override
  Future<VoidResult> setChannelProfile(int value) {
    apiLogger.i('setChannelProfile: value=$value');
    return _platform.setChannelProfile(roomUuid, value);
  }

  @override
  Future<VoidResult> setClientRole(int role) {
    apiLogger.i('setClientRole: role=$role');
    return _platform.setClientRole(roomUuid, role);
  }

  @override
  Future<VoidResult> setAudioProfile(int profile, int scenario) {
    apiLogger.i('setAudioProfile: profile=$profile, scenario=$scenario');
    return _platform.setLocalAudioProfile(roomUuid, profile, scenario);
  }

  @override
  Future<VoidResult> setRecordDeviceMute(bool mute) {
    apiLogger.i('setRecordDeviceMute: mute=$mute');
    return _platform.setRecordDeviceMute(roomUuid, mute);
  }

  @override
  Future<VoidResult> enableAudioAINS(bool enable) {
    apiLogger.i('enableAudioAINS: enable=$enable');
    return _platform.enableAudioAINS(roomUuid, enable);
  }

  @override
  Future<VoidResult> enableLoopbackRecording(bool enable) {
    apiLogger.i('enableLoopbackRecording: enable=$enable');
    return _platform.enableLoopbackRecording(roomUuid, enable);
  }

  // Future<VoidResult> adjustLoopbackRecordingSignalVolume(int volume) {
  //   apiLogger.i('adjustLoopbackRecordingSignalVolume: volume=$volume');
  //   return _platform.adjustLoopbackRecordingSignalVolume(roomUuid, volume);
  // }
}

class _NEPreviewRoomRtcControllerImpl extends NEPreviewRoomRtcController
    with _AloggerMixin {
  final _platform = PreviewRoomRtcControllerPlatform.instance;
  _NEPreviewRoomRtcControllerImpl();

  @override
  Future<VoidResult> stopPreview({bool releaseRtc = true}) {
    apiLogger.i('stopPreview: releaseRtc=$releaseRtc');
    return _platform.stopPreview(releaseRtc);
  }

  @override
  Future<VoidResult> startPreview() {
    apiLogger.i('startPreview');
    return _platform.startPreview();
  }

  @override
  Future<VoidResult> switchCamera() {
    apiLogger.i('switchCamera');
    return _platform.switchCamera();
  }

  @override
  Future<VoidResult> addBeautyFilter(String path) {
    apiLogger.i('addBeautyFilter: path$path');
    return _platform.addBeautyFilter(path);
  }

  @override
  Future<VoidResult> addBeautySticker(String path) {
    apiLogger.i('addBeautySticker: path$path');
    return _platform.addBeautySticker(path);
  }

  @override
  Future<VoidResult> enableBeauty(bool isOpenBeauty) {
    apiLogger.i('enableBeauty: isOpenBeauty$isOpenBeauty');
    return _platform.enableBeauty(isOpenBeauty);
  }

  @override
  Future<VoidResult> removeBeautyFilter() {
    apiLogger.i('removeBeautyFilter');
    return _platform.removeBeautyFilter();
  }

  @override
  Future<VoidResult> removeBeautySticker() {
    apiLogger.i('removeBeautySticker');
    return _platform.removeBeautySticker();
  }

  @override
  Future<VoidResult> setBeautyEffect(
      NERoomBeautyEffectType beautyType, double level) {
    return _platform.setBeautyEffect(beautyType, level);
  }

  @override
  Future<VoidResult> setBeautyFilterLevel(double level) {
    return _platform.setBeautyFilterLevel(level);
  }

  @override
  Future<VoidResult> startBeauty() {
    apiLogger.i('startBeauty');
    return _platform.startBeauty();
  }

  @override
  Future<VoidResult> stopBeauty() {
    apiLogger.i('stopBeauty');
    return _platform.stopBeauty();
  }

  @override
  Future<VoidResult> enableVirtualBackground(
      bool enabled, NERoomVirtualBackgroundSource backgroundSource) {
    return _platform.enableVirtualBackground(enabled, backgroundSource);
  }
}
