// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of netease_roomkit;

class _NERoomServiceImpl extends NERoomService with _AloggerMixin {
  final _platform = RoomServicePlatform.instance;

  final _contextRegistry = <String, NERoomContext>{};

  @override
  Future<VoidResult> createRoom(
      NECreateRoomParams params, NECreateRoomOptions options) {
    apiLogger.i('createRoom: $params, $options');
    return _platform.createRoom(params, options);
  }

  @override
  Future<NEResult<NERoomContext>> joinRoom(
      NEJoinRoomParams params, NEJoinRoomOptions options) {
    apiLogger.i('joinRoom: $params, $options');
    final ctx = _NERoomContextImpl(params.roomUuid);
    return _platform.joinRoom(params, options).then((value) {
      return value.map((snapshot) {
        final key = snapshot.roomUuid;
        _contextRegistry[key] = ctx;
        ctx.initState(snapshot, onDispose: () => _contextRegistry.remove(key));
        return ctx;
      }).onFailure((code, msg) {
        ctx.dispose();
      });
    });
  }

  @override
  Future<NEResult<NEPreviewRoomContext>> previewRoom(
      NEPreviewRoomParams params, NEPreviewRoomOptions options) {
    final ctx = _NEPreviewRoomContextImpl();
    return _platform.previewRoom(params, options).then((value) {
      return NEResult(code: value.code, msg: value.msg, data: ctx);
    });
  }

  @override
  NERoomContext? getRoomContext(String roomUuid) {
    return _contextRegistry[roomUuid];
  }
}
