// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of netease_roomkit;

class _NERoomWhiteboardControllerImpl extends NERoomWhiteboardController
    with _AloggerMixin {
  final _platform = RoomWhiteboardControllerPlatform.instance;
  final _NERoomContextImpl roomContext;
  @override
  final bool isSupported;
  _NERoomWhiteboardControllerImpl(this.isSupported, this.roomContext);

  @override
  String? getWhiteboardSharingUserUuid() {
    final uuid = roomContext.whiteboardSharingMember?.uuid;
    return roomContext.getMember(uuid) != null ? uuid : null;
  }

  @override
  Future<VoidResult> startWhiteboardShare() {
    apiLogger.i('startWhiteboardShare:');
    return _platform.startWhiteboardShare(roomContext.roomUuid);
  }

  @override
  Future<VoidResult> stopWhiteboardShare() {
    apiLogger.i('stopWhiteboardShare:');
    return _platform.stopWhiteboardShare(roomContext.roomUuid);
  }

  @override
  Widget createWhiteboardView({
    Set<Factory<OneSequenceGestureRecognizer>>? gestureRecognizers,
    Color? backgroundColor,
  }) {
    return NEWhiteboardView(
      roomContext.roomUuid,
      backgroundColor: backgroundColor,
      gestureRecognizers: gestureRecognizers,
    );
  }

  @override
  bool isSharingWhiteboard() {
    return roomContext.localMember.isSharingWhiteboard;
  }

  @override
  bool isWhiteboardSharing(String userId) {
    if (roomContext.localMember.uuid == userId) {
      return isSharingWhiteboard();
    } else {
      return getWhiteboardSharingUserUuid() == userId;
    }
  }

  @override
  Future<NEResult<void>> showWhiteboardTools(bool show) {
    return _platform.setEnableDraw(roomContext.roomUuid, show);
  }

  @override
  Future<VoidResult> stopMemberWhiteboardShare(String userUuid) {
    return _platform.stopMemberWhiteboardShare(roomContext.roomUuid, userUuid);
  }

  @override
  Future<VoidResult> lockCameraWithContent(int width, int height) {
    return _platform.lockCameraWithContent(roomContext.roomUuid, width, height);
  }

  @override
  Future<VoidResult> setCanvasBackgroundColor(String color) {
    return _platform.setCanvasBackgroundColor(roomContext.roomUuid, color);
  }
}
