// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of netease_roomkit;

class _NERoomKitImpl extends NERoomKit with _AloggerMixin {
  static const String fltVersion = '1.13.0';

  final _platform = RoomKitPlatform.instance;

  bool _initialized = false;

  NESDKVersions? _sdkVersions;

  @override
  Future<VoidResult> initialize(NERoomKitOptions options) async {
    apiLogger.i('initialize: version=$fltVersion');
    NEServerConfig? serverConfig = options.serverConfig;
    if (serverConfig == null && options.useAssetServerConfig) {
      try {
        final serverConfigJson =
            await rootBundle.loadString('xkit_server.config');
        serverConfig =
            NEServerConfig.fromJson(jsonDecode(serverConfigJson) as Map);
        options = NERoomKitOptions(
          appKey: options.appKey,
          useAssetServerConfig: options.useAssetServerConfig,
          serverUrl: options.serverUrl,
          serverConfig: serverConfig,
          extras: options.extras,
        );
      } catch (e, s) {
        commonLogger.e('parse asset server config error: $e\n$s');
      }
    }
    final result = await _platform.initialize(options);
    if (result.isSuccess()) {
      _initialized = true;
    }
    return result;
  }

  @override
  bool get isInitialized => _initialized;

  @override
  Future<VoidResult> switchLanguage(NERoomLanguage? language) async {
    apiLogger.i('switch language: ${language?.languageCode}');
    final code = await _platform.switchLanguage(language?.languageCode ?? '');
    return VoidResult(code: code);
  }

  @override
  Future<NESDKVersions> get sdkVersions async {
    if (_sdkVersions == null) {
      final _versions = await _platform.sdkVersions;
      _sdkVersions = NESDKVersions(
        imVersion: _versions.imVersion ?? '',
        rtcVersion: _versions.rtcVersion ?? '',
        whiteboardVersion: _versions.whiteboardVersion ?? '',
        roomKitVersion: _versions.roomKitVersion ?? '',
        fltRoomKitVersion: fltVersion,
      );
    }
    return _sdkVersions!;
  }

  @override
  final authService = _NEAuthServiceImpl();

  @override
  final roomService = _NERoomServiceImpl();

  @override
  final messageChannelService = _NEMessageServiceImpl();
}
